# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

import autopilot.exceptions
from autopilot import logging as autopilot_logging

from lomiriuitoolkit import units
from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)

# FIXME: Only left-to-right (where x=0 is leftmost) layouts are taken into
#   account. Add support for horizontal flicking with right-to-left layouts.

# Containers helpers.


def _get_visible_container_top(containers):
    containers_top = [container.globalRect.y for container in containers]
    return max(containers_top)


def _get_visible_container_bottom(containers):
    containers_bottom = [
        container.globalRect.y + container.globalRect.height
        for container in containers if container.globalRect.height > 0]
    return min(containers_bottom)


def _get_visible_container_leftmost(containers):
    containers_left = [container.globalRect.x for container in containers]
    return max(containers_left)


def _get_visible_container_rightmost(containers):
    containers_right = [
        container.globalRect.x + container.globalRect.width
        for container in containers if container.globalRect.width > 0]
    return min(containers_right)


class Scrollable(_common.LomiriUIToolkitCustomProxyObjectBase):

    @autopilot_logging.log_action(logger.info)
    def is_child_visible(self, child):
        """Determine if the child is visible.

        A child is visible if no scrolling is needed to reveal it.

        """
        containers = self._get_containers()
        return self._is_child_visible(child, containers)

    def _get_containers(self):
        """Return a list with the containers to take into account when swiping.

        The list includes this flickable and the top-most container.
        TODO add additional flickables that are between this and the top
        container. --elopio - 2014-03-22

        """
        containers = [self._get_top_container(), self]
        return containers

    def _is_child_in_visible_y_range(self, child, containers):
        """Check if the center of the child has an y-value in the visible range.

        :return: True if the center of the child is y-visible, False otherwise.
        """
        object_center_y = child.globalRect.y + child.globalRect.height // 2
        visible_top = _get_visible_container_top(containers)
        visible_bottom = _get_visible_container_bottom(containers)
        return (object_center_y >= visible_top and
                object_center_y <= visible_bottom)

    def _is_child_in_visible_x_range(self, child, containers):
        """Check if the center of the child has an x-value in the visible range.

        :return: True if the center of the child is x-visible. False otherwise.
        """
        object_center_x = child.globalRect.x + child.globalRect.width // 2
        visible_left = _get_visible_container_leftmost(containers)
        visible_right = _get_visible_container_rightmost(containers)
        return (object_center_x >= visible_left and
                object_center_x <= visible_right)

    def _is_child_visible(self, child, containers):
        """Check if the center of the child is visible.

        :return: True if the center of the child is visible, False otherwise.

        """
        return (self._is_child_in_visible_y_range(child, containers) and
                self._is_child_in_visible_x_range(child, containers))

    def _slow_drag_rate(self):
        # I found that when the flickDeceleration is 1500, the rate should be
        # 5 and that when it's 100, the rate should be 1. With those two points
        # we can get the following equation.
        # XXX The deceleration might not be linear with respect to the rate,
        # but this works for the two types of scrollables we have for now.
        # --elopio - 2014-05-08
        return (self.flickDeceleration + 250) / 350

    def _slow_drag(self, start_x, stop_x, start_y, stop_y, rate=None):
        # If we drag too fast, we end up scrolling more than what we
        # should, sometimes missing the  element we are looking for.

        # FIXME: Not all scrollables have contentX and contentY
        # (PathView does not have them), so use this only in a Flickable.
        original_content_y = self.contentY
        original_content_x = self.contentX

        if rate is None:
            rate = self._slow_drag_rate()
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y, rate=rate)

        if (self.contentY == original_content_y and
           self.contentX == original_content_x):
            raise _common.ToolkitException('Could not swipe in the flickable.')


class CannotSwipeMoreToolkitException(_common.ToolkitException):
    """Exception raised when trying to swipe more than is possible."""


class QQuickFlickable(Scrollable):

    # Swiping from below can open the deprecated toolbar or trigger the bottom
    # edge gesture. Use this margin to start a swipe that will not be that
    # close to the bottom edge.
    margin_to_swipe_from_bottom = units.gu(2)
    # Swiping from above can open the indicators or resize the window. Use this
    # margin to start a swipe that will not be that close to the top edge.
    margin_to_swipe_from_top = units.gu(2)
    # Swiping from left and right can show the launcher or open app switcher.
    margin_to_swipe_from_left = units.gu(3)
    margin_to_swipe_from_right = units.gu(3)

    @autopilot_logging.log_action(logger.info)
    def swipe_child_into_view(self, child):
        """Make the child visible.

        """
        containers = self._get_containers()
        if not self._is_child_visible(child, containers):
            self._swipe_non_visible_child_into_view(child, containers)
        else:
            logger.debug('The element is already visible.')

    @autopilot_logging.log_action(logger.info)
    def _swipe_non_visible_child_into_view(self, child, containers):
        while not self._is_child_in_visible_y_range(child, containers):
            # Check the direction of the swipe based on the position of the
            # child relative to the immediate flickable container.
            if child.globalRect.y < self.globalRect.y:
                self.swipe_to_show_more_above(containers)
            else:
                self.swipe_to_show_more_below(containers)

        while not self._is_child_in_visible_x_range(child, containers):
            if child.globalRect.x < self.globalRect.x:
                self.swipe_to_show_more_left(containers)
            else:
                self.swipe_to_show_more_right(containers)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_show_more_above(self, containers=None):
        if self.atYBeginning:
            raise CannotSwipeMoreToolkitException(
                "We are already at the top of the container.")
        else:
            self._swipe_to_show_more('above', containers)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_show_more_below(self, containers=None):
        if self.atYEnd:
            raise CannotSwipeMoreToolkitException(
                "We are already at the bottom of the container.")
        else:
            self._swipe_to_show_more('below', containers)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_show_more_left(self, containers=None):
        if self.atXBeginning:
            raise CannotSwipeMoreToolkitException(
                "We are already at the left of the container.")
        else:
            self._swipe_to_show_more('left', containers)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_show_more_right(self, containers=None):
        if self.atXEnd:
            raise CannotSwipeMoreToolkitException(
                "We are already at the right of the container.")
        else:
            self._swipe_to_show_more('right', containers)

    def _swipe_to_show_more(self, direction, containers=None):
        if containers is None:
            containers = self._get_containers()
        start_x = stop_x = self.globalRect.x + (self.globalRect.width // 2)
        start_y = stop_y = self.globalRect.y + (self.globalRect.height // 2)

        top = _get_visible_container_top(containers)
        bottom = _get_visible_container_bottom(containers)
        leftmost = _get_visible_container_leftmost(containers)
        rightmost = _get_visible_container_rightmost(containers)

        # Make the drag range be a multiple of the drag "rate" value.
        # Workarounds https://bugs.launchpad.net/mir/+bug/1399690
        rate = self._slow_drag_rate()

        # The swipes are not done from right at the top and bottom because
        # they could trigger edge gestures or resize windows.
        if direction == 'below':
            start_y = bottom - self.margin_to_swipe_from_bottom
            stop_y = start_y + (top - start_y) / rate * rate

        elif direction == 'above':
            start_y = top + self.margin_to_swipe_from_top
            stop_y = start_y + (bottom - start_y) / rate * rate

        elif direction == 'left':
            start_x = leftmost + self.margin_to_swipe_from_left
            stop_x = start_x + (rightmost - start_x) / rate * rate

        elif direction == 'right':
            start_x = rightmost - self.margin_to_swipe_from_right
            stop_x = start_x + (leftmost - start_x) / rate * rate

        else:
            raise _common.ToolkitException(
                'Invalid direction {}.'.format(direction))
        self._slow_drag(start_x, stop_x, start_y, stop_y, rate)
        self.dragging.wait_for(False)
        self.moving.wait_for(False)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_top(self):
        if not self.atYBeginning:
            containers = self._get_containers()
            while not self.atYBeginning:
                self.swipe_to_show_more_above(containers)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_bottom(self):
        if not self.atYEnd:
            containers = self._get_containers()
            while not self.atYEnd:
                self.swipe_to_show_more_below(containers)

    @autopilot_logging.log_action(logger.info)
    def swipe_to_leftmost(self):
        if not self.atXBeginning:
            containers = self._get_containers()
            while not self.atXBeginning:
                self.swipe_to_show_more_left(containers)

    def swipe_to_rightmost(self):
        if not self.atXEnd:
            containers = self._get_containers()
            while not self.atXEnd:
                self.swipe_to_show_more_right(containers)

    @autopilot_logging.log_action(logger.info)
    def pull_to_refresh(self):
        """Pulls the flickable down and triggers a refresh on it.

        :raises lomiriuitoolkit.ToolkitException: If the flickable has no pull
            to release functionality.

        """
        try:
            pull_to_refresh = self.select_single(PullToRefresh)
        except autopilot.exceptions.StateNotFoundError:
            raise _common.ToolkitException(
                'The flickable has no pull to refresh functionality.')
        self.swipe_to_top()
        self._swipe_to_middle()
        pull_to_refresh.wait_for_refresh()

    @autopilot_logging.log_action(logger.info)
    def _swipe_to_middle(self):
        start_x = stop_x = self.globalRect.x + (self.globalRect.width // 2)
        # Start just a little under the top.
        containers = self._get_containers()
        top = _get_visible_container_top(containers) + 5
        bottom = _get_visible_container_bottom(containers)

        start_y = top
        stop_y = (self.globalRect.y + bottom) // 2
        self._slow_drag(start_x, stop_x, start_y, stop_y)
        self.dragging.wait_for(False)
        self.moving.wait_for(False)

    @autopilot_logging.log_action(logger.info)
    def _cancel_pull_to_refresh(self):
        """Swipe down the list and then swipe it up to cancel the refresh."""
        # This method is kept private because it's not for the test writers to
        # call. It's to test pull to refresh. --elopio - 2014-05-22
        self.swipe_to_top()
        start_x = stop_x = self.globalRect.x + (self.globalRect.width // 2)
        # Start just a little under the top.
        containers = self._get_containers()
        top = _get_visible_container_top(containers) + 5
        bottom = _get_visible_container_bottom(containers)

        start_y = top
        stop_y = (self.globalRect.y + bottom) // 2
        self.pointing_device.move(start_x, start_y)
        self.pointing_device.press()
        self.pointing_device.move(stop_x, stop_y)
        self.pointing_device.move(start_x, start_y)
        self.pointing_device.release()


class PullToRefresh(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the PullToRefresh component."""

    def wait_for_refresh(self):
        activity_indicator = self.select_single('ActivityIndicator')
        activity_indicator.running.wait_for(False)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class Sections(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Sections Autopilot custom proxy object."""

    def __init__(self, *args):
        super().__init__(*args)
        self.listview = self.select_single(objectName='sections_listview')

    @autopilot_logging.log_action(logger.info)
    def click_section_button(self, section_index):
        """Click a section button of the Sections.

        :parameter section_index: The index of the section to click.
        :raise ToolkitException: If there is no section button with that index.

        """
        button_object_name = 'section_button_' + str(section_index)
        try:
            self.listview.click_element(button_object_name)
        except _common.ToolkitException:
            raise _common.ToolkitException(
                'Button with section index ' + str(section_index) +
                ' not found in Sections.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus

from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class Empty(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Base class to emulate swipe to delete."""

    def exists(self):
        try:
            return self.implicitHeight > 0
        except dbus.StateNotFoundError:
            return False

    def _get_confirm_button(self):
        return self.select_single(
            'QQuickItem', objectName='confirmRemovalDialog')

    @autopilot_logging.log_action(logger.info)
    def swipe_to_delete(self, direction='deprecated'):
        """Swipe the item in from left to right in order to delete it.

        :param direction: deprecated. The only direction that will cause a
            delete is left to right. Do not pass any argument to this method,
            because it will be removed in future versions.

        """
        if direction != 'deprecated':
            logger.warn(
                'The direction argument is deprecated. Now the Lomiri SDK '
                'only deletes list items when swiping from left to right. '
                'Call swipe_to_delete without arguments.')
        else:
            direction = 'right'
        if self.removable:
            self._swipe_to_delete_removable_item(direction)
        else:
            raise _common.ToolkitException(
                'The item "{0}" is not removable'.format(self.objectName))

    def _swipe_to_delete_removable_item(self, direction):
        if direction == 'left':
            raise _common.ToolkitException(
                'Only swiping to the right will cause the item to be deleted.')
        elif direction != 'right':
            raise _common.ToolkitException(
                'Invalid direction "{0}" used on swipe to delete function'
                .format(direction))

        self._drag_pointing_device_to_delete()
        if self.confirmRemoval:
            self.waitingConfirmationForRemoval.wait_for(True)
        else:
            self._wait_until_deleted()

    def _drag_pointing_device_to_delete(self):
        x, y, width, height = self.globalRect
        start_x = x + (width * 0.2)
        stop_x = x + (width * 0.8)
        start_y = stop_y = y + (height // 2)

        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)

    def _wait_until_deleted(self):
        try:
            # The item was hidden.
            self.implicitHeight.wait_for(0)
        except dbus.StateNotFoundError:
            # The item was destroyed.
            pass

    @autopilot_logging.log_action(logger.info)
    def confirm_removal(self):
        """Comfirm item removal if this was already swiped."""
        if self.waitingConfirmationForRemoval:
            deleteButton = self._get_confirm_button()
            self.pointing_device.click_object(deleteButton)
            self._wait_until_deleted()
        else:
            raise _common.ToolkitException(
                'The item "{0}" is not waiting for removal confirmation'.
                format(self.objectName))


class Base(Empty):
    pass


class Expandable(Empty):

    @autopilot_logging.log_action(logger.info)
    def expand(self):
        """Expand an expandable list item.

        If the item is already expanded, no action will be executed.
        If you want to check if the item is expanded, you can use the
        ``expanded`` property.

        """
        if not self.expanded:
            self._click_always_visible_section()
            self.expanded.wait_for(True)
            self.height.wait_for(self.expandedHeight)
        else:
            logger.debug('The item is already expanded.')

    @autopilot_logging.log_action(logger.debug)
    def _click_always_visible_section(self):
        self.pointing_device.move(
            self.globalRect.x + self.globalRect.width // 2,
            self.globalRect.y + self.collapsedHeight // 2)
        self.pointing_device.click()

    @autopilot_logging.log_action(logger.info)
    def collapse(self):
        """Collapse an expandable list item.

        If the item is already collapsed, no action will be executed.
        If you want to check if the item is expanded, you can use the
        ``expanded`` property.

        """
        if self.expanded:
            self._click_always_visible_section()
            self.expanded.wait_for(False)
            self.height.wait_for(self.collapsedHeight)
        else:
            logger.debug('The item is already collapsed.')


class Standard(Empty):
    pass


class ItemSelector(Empty):
    pass


class SingleControl(Empty):
    pass


class MultiValue(Base):
    pass


class SingleValue(Base):
    pass


class Subtitled(Base):
    pass



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the QML Slider component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class SliderTests(TavastiaTestCase):
    """Tests for Slider component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"SliderTests")

    def test_can_select_slider(self):
        """Must be able to select the Qml Slider component."""

        obj = self.app.select_single('Slider')
        self.assertThat(obj, Not(Is(None)))
    
    def test_can_set_live(self):
        """Must be able to set live"""

        obj = self.app.select_single('Slider')
        btn = self.app.select_single('Button', text="live")

        # default value is False
        self.assertThat(obj.live, Equals(False))

        self.mouse.move_to_object(btn)
        self.mouse.click()

        # we should be able to set the value within the QML, check Button::onClicked
        self.assertThat(obj.live, Eventually(Equals(True)))

    def test_can_set_minimum_and_maximum(self):
        """Must be able to set minimum and maximum values"""

        obj = self.app.select_single('Slider')
        btn = self.app.select_single('Button', text="minmax_values")

        self.assertThat(obj.minimumValue, Equals(0))
        self.assertThat(obj.maximumValue, Equals(11))

        old_minimumValue = obj.minimumValue
        old_maximumValue = obj.maximumValue

        self.mouse.move_to_object(btn)
        self.mouse.click()

        # we should be able to set the value within the QML, check Button::onClicked
        self.assertThat(obj.minimumValue, Eventually(Equals(old_minimumValue+1)))
        self.assertThat(obj.maximumValue, Eventually(Equals(old_maximumValue+10)))

    def test_can_set_value(self):
        """Must be able to set value"""

        obj = self.app.select_single('Slider')
        btnreset = self.app.select_single('Button', text="reset values")
        btnvalue = self.app.select_single('Button', text="inc value")

        # lets reset values
        self.mouse.move_to_object(btnreset)
        self.mouse.click()
        
        # read current values
        self.assertThat(obj.maximumValue, Eventually(Equals(10)))
        self.assertThat(obj.minimumValue, Eventually(Equals(-10)))

        value = obj.value
        maximumValue = obj.maximumValue
        minimumValue = obj.minimumValue

        # start the qml timer which should increase the value
        self.mouse.move_to_object(btnvalue)
        self.mouse.click()

        # the value should have changed but maximumValue and minimumValue should be the same
        self.assertThat(obj.value, Eventually(Equals(maximumValue)))
        self.assertThat(obj.maximumValue, Eventually(Equals(maximumValue)))
        self.assertThat(obj.minimumValue, Eventually(Equals(minimumValue)))




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Emulators was the old name for the custom proxy objects."""

import logging

from lomiriuitoolkit._custom_proxy_objects import (
    check_autopilot_version,
    get_keyboard,
    get_pointing_device,
    CheckBox,
    Header,
    Dialog,
    UCListItem,
    MainView,
    OptionSelector,
    QQuickFlickable,
    QQuickListView,
    TabBar,
    Tabs,
    TextField,
    Toolbar,
    ToolkitException as ToolkitEmulatorException,
    LomiriUIToolkitCustomProxyObjectBase as LomiriUIToolkitEmulatorBase,
)
from lomiriuitoolkit._custom_proxy_objects.listitems import (
    Base,
    Empty,
    ItemSelector,
    MultiValue,
    SingleControl,
    SingleValue,
    Standard,
    Subtitled
)
from lomiriuitoolkit._custom_proxy_objects.popups import (
    TextInputPopover,
    ActionSelectionPopover,
    ComposerSheet
)


logger = logging.getLogger(__name__)


logger.warning(
    'The lomiriuitoolkit.emulators module is deprecated. Import the autopilot '
    'helpers from the top-level lomiriuitoolkit module.')


__all__ = [
    'check_autopilot_version',
    'get_keyboard',
    'get_pointing_device',
    'TextInputPopover',
    'ActionSelectionPopover',
    'Base',
    'CheckBox',
    'ComposerSheet',
    'Empty',
    'UCListItem',
    'Header',
    'Dialog',
    'ItemSelector',
    'MainView',
    'MultiValue',
    'OptionSelector',
    'QQuickFlickable',
    'QQuickListView',
    'SingleControl',
    'SingleValue',
    'Standard',
    'Subtitled',
    'TabBar',
    'Tabs',
    'TextField',
    'Toolbar',
    'ToolkitEmulatorException',
    'LomiriUIToolkitEmulatorBase',
]



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Gallery"""

import testscenarios

import lomiriuitoolkit
from lomiriuitoolkit import lomiri_scenarios
from lomiriuitoolkit.tests import gallery


class GalleryAppTestCase(gallery.GalleryTestCase):
    """Generic tests for the Gallery"""

    scenarios = lomiri_scenarios.get_device_simulation_scenarios()

    def test_select_main_view_must_return_main_window_emulator(self):
        main_view = self.main_view
        self.assertIsInstance(main_view, lomiriuitoolkit.MainView)

    def test_slider(self):
        self.open_page('slidersElement')

        item_data = [
            ["slider_standard"],
            ["slider_live"],
            ["slider_range"]
        ]

        for data in item_data:
            objName = data[0]
            self.getObject(objName)
            self.tap(objName)

            # TODO: move slider value

    def test_progress_and_activity(self):
        self.open_page('progressBarsElement')

        item_data = [
            ["progressbar_standard"],
            ["progressbar_indeterminate"],
            ["activityindicator_standard"]
        ]

        for data in item_data:
            objName = data[0]
            self.getObject(objName)
            self.tap(objName)

            # TODO: check for properties

    def test_lomirishape(self):
        # Flaky test case
        # FIXME: https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1308979
        return

        self.open_page('lomiriShapesElement')

        item_data = [
            ["lomirishape_color_hex"],
            ["lomirishape_color_lightblue"],
            ["lomirishape_color_darkgray"],
            ["lomirishape_image"],
            ["lomirishape_radius_small"],
            ["lomirishape_radius_medium"],
            ["lomirishape_sizes_15_6"],
            ["lomirishape_sizes_10_14"]
        ]

        for data in item_data:
            objName = data[0]
            self.getObject(objName)


class OpenPagesTestCase(gallery.GalleryTestCase):

    names = [
        'navigation', 'toggles', 'buttons', 'sliders', 'textinputs',
        'optionSelectors', 'pickers', 'progressBars', 'lomiriShapes', 'icons',
        'labels', 'listItems', 'lomiriListView', 'dialogs', 'popovers',
        'sheets', 'animations'
    ]

    pages_scenarios = [
        (name, dict(
            element_name=name+'Element',
            template_name=name+'Template'))
        for name in names
    ]

    scenarios = testscenarios.multiply_scenarios(
        lomiri_scenarios.get_device_simulation_scenarios(),
        pages_scenarios)

    def test_open_page(self):
        self.open_page(self.element_name)
        self.main_view.wait_select_single(objectName=self.template_name)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging
import os

import fixtures
from testtools.matchers import Contains
from autopilot import introspection

import lomiriuitoolkit
from lomiriuitoolkit import tests


class HideShowTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path,
        'test_header.HideShowTestCase.qml')

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()

    def test_ensure_header_visible_must_show_it_when_not_visible(self):
        """Test that header.ensure_visible() shows the header."""

        # NOTE: Using the internal _is_visible() function here.
        # It is not made public because besides testing the
        # ensure_visible() function, there is no use for it.
        self.assertEquals(self.header._is_visible(), True)
        # Scroll down to hide the header
        self.list_view = self.main_view.select_single(
            lomiriuitoolkit.QQuickListView, objectName='testListView')
        self.list_view.click_element('testListElement19')
        self.assertEquals(self.header._is_visible(), False)
        self.header.ensure_visible()
        self.assertEquals(self.header._is_visible(), True)


class ActionsTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    tools_test_qml_file_path = os.path.join(
        dir_path, 'test_header.ToolsTestCase.qml')
    actions_test_qml_file_path = os.path.join(
        dir_path, 'test_header.ActionsTestCase.qml')

    scenarios = [
        ('deprecated tools',
            dict(test_qml_file_path=tools_test_qml_file_path)),
        ('actions',
            dict(test_qml_file_path=actions_test_qml_file_path))
    ]

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()
        self.label = self.app.select_single(objectName='clicked_label')
        self.assertEqual(self.label.text, 'No button clicked.')

    def test_header_custom_proxy_object(self):
        self.assertIsInstance(self.header, lomiriuitoolkit.AppHeader)
        self.assertTrue(self.header.visible)
        self.assertEqual(self.header.title, "Test title")

    def test_click_header_action_button(self):
        self.header.click_action_button('action0')
        self.assertEqual(self.label.text, 'Button 0 clicked.')

    def test_click_header_overflow_action_button(self):
        # custom back button and first action button go in the header
        # and the others in the overflow.
        self.header.click_action_button('action3')
        self.assertEqual(self.label.text, 'Button 3 clicked.')

    def test_click_unexisting_header_action_button(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.header.click_action_button,
            'unexisting')
        self.assertEqual(
            str(error),
            'Button not found in header or overflow')

    # back button and tabs button are tested in pagestack and tabs tests
    def test_click_custom_back_button(self):
        self.header.click_custom_back_button()
        self.assertEqual(self.label.text, 'Cancel button clicked.')

    def test_click_header_action_button_with_hidden_header(self):
        bottom_label = self.main_view.select_single(objectName='end_label')
        bottom_label.swipe_into_view()
        self.assertFalse(self.header._is_visible())
        self.header.click_action_button('action0')
        self.assertEqual(self.label.text, 'Button 0 clicked.')


class SectionsTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_header.SectionsTestCase.qml')

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()
        # initially, section 0 is selected
        self.assertEqual(self.header.get_selected_section_index(), 0)

    def test_select_sections(self):
        for index in [1, 0, 2]:
            self.header.switch_to_section_by_index(index)
            self.assertEqual(self.header.get_selected_section_index(), index)

    def test_select_sections_with_sections_disabled(self):
        sectionsEnabledSwitch = self.app.select_single(
            'CheckBox', objectName='sections_enabled_switch')
        sectionsEnabledSwitch.uncheck()
        for index in [1, 0, 2]:
            self.header.switch_to_section_by_index(index)
            # cannot change section by tapping the section name in divider
            self.assertEqual(self.header.get_selected_section_index(), 0)


class DeprecatedHeaderSectionsTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_header.DeprecatedHeaderSectionsTestCase.qml')

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()

    def test_get_selection_index(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.header.get_selected_section_index)
        self.assertEqual(
            str(error),
            'Old header has no sections')

    def test_select_sections(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.header.switch_to_section_by_index, 1)
        self.assertEqual(
            str(error),
            'Old header has no sections')

    def test_select_sections_with_sections_disabled(self):
        sectionsEnabledSwitch = self.app.select_single(
            'CheckBox', objectName='sections_enabled_switch')
        sectionsEnabledSwitch.uncheck()
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.header.switch_to_section_by_index, 1)
        self.assertEqual(
            str(error),
            'Old header has no sections')


class CustomMainView(lomiriuitoolkit.MainView):
    """Autopilot helper for a custom main view."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        state_name = introspection.get_classname_from_path(path)
        class_name = cls.__name__.encode('utf-8')
        return state_name == class_name


class HeaderInCustomMainViewTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_header.HeaderInCustomMainViewTestCase.qml')

    @property
    def main_view(self):
        return self.app.select_single(objectName='customMainView')

    def test_get_header_from_custom_main_view(self):
        """Test that we can get the header from a custom main view.

        This prevents a regression of http://pad.lv/1324556.

        """
        header = self.main_view.get_header()
        self.assertIsInstance(header, lomiriuitoolkit.AppHeader)


class DeprecatedHeaderTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_header.DeprecatedHeaderTestCase.qml')

    @property
    def main_view(self):
        return self.app.select_single('QQuickItem', objectName='main')

    def test_get_deprecated_header_must_log_deprecation_warning(self):
        fake_logger = fixtures.FakeLogger(level=logging.WARNING)
        self.useFixture(fake_logger)
        self.main_view.select_single(lomiriuitoolkit.Header)
        self.assertThat(
            fake_logger.output,
            Contains(
                'Header is an internal QML component of Lomiri.Components and '
                'its API may change or be removed at any moment. Please use '
                'MainView and Page instead.'))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import datetime
import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus

from lomiriuitoolkit._custom_proxy_objects import (
    _common,
    _qquicklistview
)


logger = logging.getLogger(__name__)


class DatePicker(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the DatePicker component."""

    @autopilot_logging.log_action(logger.info)
    def pick_date(self, date):
        """Pick a date from the date picker.

        :parameter date: The date to pick.
        :type date: An object with year, month and day attributes, like
            python's datetime.date.
        :raises lomiriuitoolkit.ToolkitException if the mode of the picker
            doesn't let select a date.

        """
        if not self._is_date_picker():
            raise _common.ToolkitException(
                "Can't pick date. The picker mode is: {!r}.".format(self.mode))
        if 'Years' in self.mode:
            self._pick_year(date.year)
            self.year.wait_for(date.year)
        if 'Month' in self.mode:
            # Python's date object starts at one. The model in the date picker
            # at 0.
            self._pick_month(date.month - 1)
            self.month.wait_for(date.month - 1)
        if 'Day' in self.mode:
            self._pick_day(date.day)
            self.day.wait_for(date.day)

    def _is_date_picker(self):
        mode = self.mode
        if 'Years' in mode or 'Months' in mode or 'Days' in mode:
            return True
        else:
            return False

    @autopilot_logging.log_action(logger.info)
    def _pick_year(self, year):
        picker = self.select_single(
            'Picker', objectName='PickerRow_YearPicker')
        list_view = picker.select_single(
            _qquicklistview.QQuickListView, objectName='Picker_Linear')
        self._pick_value(self.year, year, list_view)

    @autopilot_logging.log_action(logger.info)
    def _pick_month(self, month):
        self._pick_value_from_path_view('Month', self.month, month)

    @autopilot_logging.log_action(logger.info)
    def _pick_day(self, day):
        # Python's date object starts at one. The model in the date picker
        # at 0.
        self._pick_value_from_path_view('Day', self.day - 1, day - 1)

    def _pick_value_from_path_view(self, type_, current_value, new_value):
        picker = self.select_single(
            'Picker', objectName='PickerRow_{}Picker'.format(type_))
        path_view = picker.select_single(
            QQuickPathView, objectName='Picker_WrapAround')
        self._pick_value(current_value, new_value, path_view)

    def _pick_value(self, current_value, new_value, scrollable):
        if new_value > current_value:
            direction = 'below'
        elif new_value < current_value:
            direction = 'above'
        else:
            logger.debug('The value is already selected.')
            return
        scrollable.click_element(
            object_name='PickerRow_PickerLabel{}'.format(new_value),
            direction=direction)

    def get_date(self):
        """Return the currently selected date.

        :return: a python datetime.date object with the selected date.

        """
        # Python's date object starts at one. The model in the date picker
        # at 0.
        return datetime.date(self.year, self.month + 1, self.day)

    @autopilot_logging.log_action(logger.info)
    def pick_time(self, time):
        """Pick a time from the date picker.

        :parameter time: The time to pick.
        :type time: An object with hour, minute and second attributes, like
            python's datetime.time.
        :raises lomiriuitoolkit.ToolkitException if the mode of the picker
            doesn't let select a time.

        """
        if not self._is_time_picker():
            raise _common.ToolkitException(
                "Can't pick time. The picker mode is: {!r}.".format(self.mode))
        # Workaround https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1346669
        # By setting seconds, then minutes, then hours, erratic behavoir
        # can be dealt with
        if 'Seconds' in self.mode:
            self._pick_second(time.second)
            self.seconds.wait_for(time.second)
        if 'Minutes' in self.mode:
            self._pick_minute(time.minute)
            self.minutes.wait_for(time.minute)
        if 'Hours' in self.mode:
            self._pick_hour(time.hour)
            self.hours.wait_for(time.hour)

    def _is_time_picker(self):
        mode = self.mode
        if 'Hours' in mode or 'Minutes' in mode or 'Seconds' in mode:
            return True
        else:
            return False

    @autopilot_logging.log_action(logger.info)
    def _pick_hour(self, hour):
        self._pick_value_from_path_view('Hours', self.hours, hour)

    @autopilot_logging.log_action(logger.info)
    def _pick_minute(self, minute):
        self._pick_value_from_path_view('Minutes', self.minutes, minute)

    @autopilot_logging.log_action(logger.info)
    def _pick_second(self, second):
        self._pick_value_from_path_view('Seconds', self.seconds, second)

    def get_time(self):
        """Return the currently selected time.

        :return: a python datetime.time object with the selected time.

        """
        return datetime.time(self.hours, self.minutes, self.seconds)

# Containers helpers.


def _get_visible_container_top(containers):
    containers_top = [container.globalRect.y for container in containers]
    return max(containers_top)


def _get_visible_container_bottom(containers):
    containers_bottom = [
        container.globalRect.y + container.globalRect.height
        for container in containers if container.globalRect.height > 0]
    return min(containers_bottom)


class Scrollable(_common.LomiriUIToolkitCustomProxyObjectBase):

    @autopilot_logging.log_action(logger.info)
    def is_child_visible(self, child):
        """Determine if the child is visible.

        A child is visible if no scrolling is needed to reveal it.

        """
        containers = self._get_containers()
        return self._is_child_visible(child, containers)

    def _get_containers(self):
        """Return a list with the containers to take into account when swiping.

        The list includes this flickable and the top-most container.
        TODO add additional flickables that are between this and the top
        container. --elopio - 2014-03-22

        """
        containers = [self._get_top_container(), self]
        return containers

    def _is_child_visible(self, child, containers):
        """Check if the center of the child is visible.

        :return: True if the center of the child is visible, False otherwise.

        """
        object_center = child.globalRect.y + child.globalRect.height // 2
        visible_top = _get_visible_container_top(containers)
        visible_bottom = _get_visible_container_bottom(containers)
        return (object_center >= visible_top and
                object_center <= visible_bottom)

    def _slow_drag_rate(self):
        # I found that when the flickDeceleration is 1500, the rate should be
        # 5 and that when it's 100, the rate should be 1. With those two points
        # we can get the following equation.
        # XXX The deceleration might not be linear with respect to the rate,
        # but this works for the two types of scrollables we have for now.
        # --elopio - 2014-05-08
        return (self.flickDeceleration + 250) / 350

    def _slow_drag(self, start_x, stop_x, start_y, stop_y, rate=None):
        # If we drag too fast, we end up scrolling more than what we
        # should, sometimes missing the  element we are looking for.

        # FIXME: QQuickPathView has no contentY property, but it was added
        # to the PathView used inside the Picker in Picker.qml
        original_content_y = self.contentY

        if rate is None:
            rate = self._slow_drag_rate()
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y, rate=rate)

        if self.contentY == original_content_y:
            raise _common.ToolkitException('Could not swipe in the flickable.')


class QQuickPathView(Scrollable):

    # TODO make it more general and move it to its own module.
    # --elopio - 2014-05-06

    @autopilot_logging.log_action(logger.info)
    def click_element(self, object_name, direction='below'):
        try:
            element = self.select_single(objectName=object_name)
        except dbus.StateNotFoundError:
            # The element might be on a part of the list that hasn't been
            # created yet. We have to search for it scrolling.
            element = self._find_element(object_name, direction)
        self.swipe_child_into_view(element)
        self.pointing_device.click_object(element)

    @autopilot_logging.log_action(logger.info)
    def _find_element(self, object_name, direction):
        containers = self._get_containers()
        for index in range(self.count):
            if direction == 'below':
                swipe_method = self._swipe_to_show_one_more_below
            elif direction == 'above':
                swipe_method = self._swipe_to_show_one_more_above
            else:
                raise _common.ToolkitException(
                    'Invalid direction: {}'.format(direction))

            swipe_method(containers)

            try:
                return self.select_single(objectName=object_name)
            except dbus.StateNotFoundError:
                pass
        raise _common.ToolkitException(
            'List element with objectName "{}" not found.'.format(object_name))

    @autopilot_logging.log_action(logger.info)
    def _swipe_to_show_one_more_above(self, containers):
        self._swipe_to_show_one_more('above', containers)

    @autopilot_logging.log_action(logger.info)
    def _swipe_to_show_one_more_below(self, containers):
        self._swipe_to_show_one_more('below', containers)

    def _swipe_to_show_one_more(self, direction, containers):
        start_x = stop_x = self.globalRect.x + (self.globalRect.width // 2)
        center_y = self.globalRect.y + (self.globalRect.height // 2)
        # XXX This assumes all the children are of the same height
        child = self.get_children_by_type('PickerDelegate')[0]
        top = center_y - (child.globalRect.height // 2)
        bottom = center_y + (child.globalRect.height // 2)
        if direction == 'below':
            start_y = bottom
            stop_y = top
        elif direction == 'above':
            start_y = top
            stop_y = bottom
        else:
            raise _common.ToolkitException(
                'Invalid direction {}.'.format(direction))
        self._slow_drag(start_x, stop_x, start_y, stop_y)
        self.dragging.wait_for(False)
        self.moving.wait_for(False)

    @autopilot_logging.log_action(logger.info)
    def swipe_child_into_view(self, child):
        """Make the child visible.

        Currently it works only when the object needs to be swiped vertically.
        TODO implement horizontal swiping. --elopio - 2014-03-21

        """
        containers = self._get_containers()
        if not self._is_child_visible(child, containers):
            self._swipe_non_visible_child_into_view(child, containers)
        else:
            logger.debug('The element is already visible.')

    @autopilot_logging.log_action(logger.info)
    def _swipe_non_visible_child_into_view(self, child, containers):
        while not self._is_child_visible(child, containers):
            # Check the direction of the swipe based on the position of the
            # child relative to the immediate flickable container.
            if child.globalRect.y < self.globalRect.y:
                self._swipe_to_show_one_more_above(containers)
            else:
                self._swipe_to_show_one_more_below(containers)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import datetime

import lomiriuitoolkit
from lomiriuitoolkit import pickers, tests


class DatePickerBaseTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_date_picker.DatePickerBaseTestCase.qml')

    def setUp(self):
        super().setUp()
        self.date_picker = self.main_view.select_single(
            pickers.DatePicker, objectName='datePicker')
        self.time_picker = self.main_view.select_single(
            pickers.DatePicker, objectName='timePicker')


class DatePickerTestCase(DatePickerBaseTestCase):

    def test_select_date_picker_must_return_custom_proxy_object(self):
        self.assertIsInstance(
            self.date_picker, pickers.DatePicker)

    def test_pick_date_on_time_picker_must_raise_exception(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.time_picker.pick_date,
            'dummy')
        self.assertEqual(
            str(error),
            "Can't pick date. The picker mode is: {!r}.".format(
                self.time_picker.mode))

    def test_pick_time_on_date_picker_must_raise_exception(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.date_picker.pick_time,
            'dummy')
        self.assertEqual(
            str(error),
            "Can't pick time. The picker mode is: {!r}.".format(
                self.date_picker.mode))

    def test_swipe_to_show_one_more_below_must_select_next_index(self):
        """Test that we don't end up swiping more than needed.

        This would cause us to miss the element we are looking for, and to have
        to swipe many times in order to finally click it.

        """
        picker = self.main_view.select_single(
            'Picker', objectName='PickerRow_DayPicker')
        path_view = picker.select_single(
            pickers.QQuickPathView, objectName='Picker_WrapAround')
        current_index = path_view.currentIndex

        path_view._swipe_to_show_one_more_below(path_view._get_containers())

        self.assertEqual(path_view.currentIndex, current_index + 1)

    def test_swipe_to_show_one_more_above_must_select_previous_index(self):
        """Test that we don't end up swiping more than needed.

        This would cause us to miss the element we are looking for, and to have
        to swipe many times in order to finally click it.

        """
        picker = self.main_view.select_single(
            'Picker', objectName='PickerRow_DayPicker')
        path_view = picker.select_single(
            pickers.QQuickPathView, objectName='Picker_WrapAround')
        current_index = path_view.currentIndex

        path_view._swipe_to_show_one_more_above(path_view._get_containers())

        self.assertEqual(path_view.currentIndex, current_index - 1)


class PickDateFromDatePickerTestCase(DatePickerBaseTestCase):

    SELECTED_YEAR = 2010
    SELECTED_MONTH = 6  # June
    SELECTED_DAY = 15

    scenarios = [
        ('higher year', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR + 10, SELECTED_MONTH, SELECTED_DAY)}),
        ('lower year', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR - 10, SELECTED_MONTH, SELECTED_DAY)}),
        ('higher month', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR, SELECTED_MONTH + 4, SELECTED_DAY)}),
        ('lower month', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR, SELECTED_MONTH - 4, SELECTED_DAY)}),
        ('higher day', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR, SELECTED_MONTH, SELECTED_DAY + 10)}),
        ('lower day', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR, SELECTED_MONTH, SELECTED_DAY - 10)}),
        ('change all values', {
            'date_to_pick': datetime.date(
                SELECTED_YEAR - 10, SELECTED_MONTH + 4, SELECTED_DAY - 10)}),
    ]

    def test_pick_date(self):
        """Test that picking a date updates the picker."""
        self.date_picker.pick_date(self.date_to_pick)
        self.assertEqual(self.date_picker.get_date(), self.date_to_pick)


class PickTimeFromDatePickerTestCase(DatePickerBaseTestCase):

    SELECTED_HOUR = 12
    SELECTED_MINUTE = 30
    SELECTED_SECOND = 30

    scenarios = [
        ('higher hour', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR + 6, SELECTED_MINUTE, SELECTED_SECOND)}),
        ('lower hour', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR - 6, SELECTED_MINUTE, SELECTED_SECOND)}),
        ('higher minute', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR, SELECTED_MINUTE + 10, SELECTED_SECOND)}),
        ('lower minute', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR, SELECTED_MINUTE - 10, SELECTED_SECOND)}),
        ('higher second', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR, SELECTED_MINUTE, SELECTED_SECOND + 10)}),
        ('lower second', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR, SELECTED_MINUTE, SELECTED_SECOND - 10)}),
        ('change all values', {
            'time_to_pick': datetime.time(
                SELECTED_HOUR + 6, SELECTED_MINUTE - 10,
                SELECTED_SECOND + 10)}),
    ]

    def test_pick_time(self):
        self.time_picker.pick_time(self.time_to_pick)
        self.assertEqual(self.time_picker.get_time(), self.time_to_pick)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from unittest import mock

import os

import testscenarios
from autopilot import platform
from autopilot.introspection import dbus

import lomiriuitoolkit
from lomiriuitoolkit import tests


class QQuickListViewTestCase(tests.QMLFileAppTestCase):
    """Test clicking on elements in the ListView

    The ListView may need to be scrolled forward or backwards before
    the element can be clicked. In a standard vertical ListView,
    'forward' means 'down' and 'backward' means 'up. In a horizontal
    ListView, 'forward' means to the right, and 'backward' means to
    the left.

    TODO: Add support for right-to-left layouts.
    """

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    uitk10v_qml_file_path = os.path.join(
        dir_path, 'test_listview.QQuickListViewTestCase.Uitk10.vertical.qml')
    uitk13v_qml_file_path = os.path.join(
        dir_path, 'test_listview.QQuickListViewTestCase.Uitk13.vertical.qml')
    uitk13h_qml_file_path = os.path.join(
        dir_path, 'test_listview.QQuickListViewTestCase.Uitk13.horizontal.qml')

    scenarios = [
        ('UITK 1.0 vertical',
            dict(test_qml_file_path=uitk10v_qml_file_path)),
        ('UITK 1.3 vertical',
            dict(test_qml_file_path=uitk13v_qml_file_path)),
        ('UITK 1.3 horizontal',
            dict(test_qml_file_path=uitk13h_qml_file_path))
    ]

    def setUp(self):
        super().setUp()
        self.list_view = self.main_view.select_single(
            lomiriuitoolkit.QQuickListView, objectName='testListView')
        self.label = self.main_view.select_single(
            objectName='clickedLabel')
        self.assertEqual(self.label.text, 'No element clicked.')

    def test_qquicklistview_custom_proxy_object(self):
        self.assertIsInstance(self.list_view, lomiriuitoolkit.QQuickListView)

    def _click_element(self, object_name):
        """Click the element with the given object name and verify
           that the label text was updated.
        """
        self.list_view.click_element(object_name)
        self.assertEqual(self.label.text, object_name)

    def test_click_element_in_view(self):
        self.assertTrue(self.list_view._is_element_visible('testListElement0'))
        self._click_element('testListElement0')

    def test_click_cached_element_outside_view_after(self):
        # Click the first element out of view to make sure we are not scrolling
        # towards the end at once.
        i = 0
        while i < self.list_view.count:
            objectName = 'testListElement' + str(i)
            if not self.list_view._is_element_visible(objectName):
                break
            i = i + 1

        # _is_element_visible() checks that the center of the element is not
        # visible. To make sure the element is fully invisible, take the
        # next one.
        i = i + 1
        objectName = 'testListElement' + str(i)

        if i == self.list_view.count:
            raise lomiriuitoolkit.ToolkitException(
                'Example program has no invisible item that is not cached.')

        self.assertTrue(self.list_view._is_element_cached(objectName))
        self._click_element(objectName)

    def test_click_element_outside_view_before(self):
        self.list_view.click_element('testListElement9')

        self.assertFalse(
            self.list_view._is_element_visible('testListElement4'))

        self._click_element('testListElement4')

    def test_click_uncached_element_outside_view_after(self):
        objectName = 'testListElement19'
        self.assertFalse(self.list_view._is_element_cached(objectName))
        self._click_element(objectName)

    def test_click_element_from_first_page_deleted_when_swiping_forward(self):
        """Test that no more swiping is done after finding the element.

        This is a regression test for bug http://pad.lv/1342521 that caused
        us to swipe down after finding an element on the first page of the
        list. If we were lucky, the element was still cached so we just ended
        up doing two extra swipes. If not, then the element will be deleted
        from the tree and the helper failed.

        """
        # Swipe forward.
        self.list_view.click_element('testListElement19')

        objectName = 'testListElement1'
        self.assertFalse(self.list_view._is_element_cached(objectName))

        with mock.patch.object(
                self.list_view,
                'swipe_to_show_more_below') as mock_swipe_down:
            with mock.patch.object(
                    self.list_view,
                    'swipe_to_show_more_right') as mock_swipe_right:
                self._click_element(objectName)

        self.assertFalse(mock_swipe_down.called)
        self.assertFalse(mock_swipe_right.called)

    def test_click_unexisting_element(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.list_view.click_element,
            'unexisting')
        self.assertEqual(
            str(error), 'List element with objectName "unexisting" not found.')


class QQuickListViewOutOfViewTestCase(tests.QMLFileAppTestCase):
    """Test that we can make elements visible when the list is out of view."""

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_qquicklistview.QQuickListViewOutOfViewTestCase.qml')

    def setUp(self):
        super().setUp()
        self.list_view = self.main_view.select_single(
            lomiriuitoolkit.QQuickListView, objectName='testListView')
        self.label = self.main_view.select_single(
            'Label', objectName='clickedLabel')
        self.assertEqual(self.label.text, 'No element clicked.')

    def test_click_element_outside_view_below(self):
        """Test that we can click an element that's out of view below.

        The list is also out of view, so we must scroll from the bottom of the
        main view.

        """
        # Test for http://pad.lv/1275060.
        # Click the first element out of view to make sure we are not scrolling
        # to the bottom at once.
        self.assertFalse(
            self.list_view._is_element_visible('testListElement9'))

        self.list_view.click_element('testListElement9')
        self.assertEqual(self.label.text, 'testListElement9')


class QQuickListViewDraggingBaseTestCase(tests.QMLFileAppTestCase):

    def setUp(self):
        path = os.path.abspath(__file__)
        dir_path = os.path.dirname(path)
        self.test_qml_file_path = os.path.join(
            dir_path, self.qml_file_name)

        super().setUp()
        self.list_view = self.main_view.select_single(
            lomiriuitoolkit.QQuickListView, objectName='test_view')


class QQuickListViewDraggingTestCase(QQuickListViewDraggingBaseTestCase):

    qml_file_name = 'test_listitem.ListViewWithLiveDraggingTestCase.qml'

    def test_long_press_must_enable_drag_mode(self):
        list_item = self.list_view.select_single(
            'UCListItem', objectName='listitem0')
        self.assertFalse(list_item.dragMode)
        self.list_view._enable_drag_mode()
        self.assertTrue(list_item.dragMode)


class QQuickListViewReorderingTestCase(QQuickListViewDraggingBaseTestCase):

    dragging_scenarios = [
        ('with live dragging', {
            'qml_file_name': (
                'test_listitem.ListViewWithLiveDraggingTestCase.qml')}),
        ('without live dragging', {
            'qml_file_name': (
                'test_listitem.ListViewWithoutLiveDraggingTestCase.qml')}),
    ]
    reorder_scenarios = [
        ('both items visible, to bottom', {'from_index': 0, 'to_index': 1}),
        ('both items visible, to top', {'from_index': 1, 'to_index': 0}),
        ('both items visible, to bottom, first non visible', {
            'from_index': 0, 'to_index': 7}),
        ('to item not visible, to top', {'from_index': 15, 'to_index': 0}),
        ('to item not visible, to middle down', {
            'from_index': 0, 'to_index': 15}),
        ('to item not visible, to bottom', {'from_index': 0, 'to_index': 24})
    ]
    scenarios = testscenarios.multiply_scenarios(
        dragging_scenarios, reorder_scenarios)

    def _find_item(self, index):
        object_name = 'listitem{}'.format(index)
        try:
            return self.list_view.select_single(objectName=object_name)
        except dbus.StateNotFoundError:
            return self.list_view._find_element(object_name)

    def _get_item_text(self, index):
        item = self._find_item(index)
        return item.select_single('Label').text

    def test_drag_item_must_reorder_list(self):
        if platform.model() != 'Desktop':
            self.skipTest(
                'Drag does not work on the phone because of bug #1266601')

        original_from_text = self._get_item_text(self.from_index)

        self.list_view.drag_item(self.from_index, self.to_index)

        self.assertEqual(
            original_from_text, self._get_item_text(self.to_index))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os

import lomiriuitoolkit
from autopilot.introspection import dbus
from lomiriuitoolkit import tests


class ListItemTestCase(tests.QMLFileAppTestCase):
    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_listitem.ListItemTestCase.qml')

    def setUp(self):
        super().setUp()
        self.list_view = self.main_view.select_single(
            lomiriuitoolkit.QQuickListView, objectName='test_view')
        self.test_listitem = self.main_view.select_single(
            'UCListItem', objectName='listitem0')
        self.test_page = self.main_view.select_single(
            objectName='test_page')
        header = self.test_page.select_single(
            'PageHeader', objectName='test_header')
        self.assertEqual(header.title, 'No action triggered')

    def test_trigger_delete(self):
        self.test_listitem.trigger_leading_action('delete_action')
        self.assertEqual(self.test_page.title,
                         'delete_action action triggered')

    def test_trigger_search(self):
        self.test_listitem.trigger_trailing_action('search_action')
        self.assertEqual(self.test_page.title,
                         'search_action action triggered')

    def test_trigger_edit(self):
        self.test_listitem.trigger_trailing_action('edit_action')
        self.assertEqual(self.test_page.title,
                         'edit_action action triggered')

    def test_trigger_email(self):
        self.test_listitem.trigger_trailing_action('email_action')
        self.assertEqual(self.test_page.title,
                         'email_action action triggered')

    def test_trigger_all_actions(self):
        self.test_listitem.trigger_leading_action('delete_action')
        self.assertEqual(self.test_page.title,
                         'delete_action action triggered')

        self.test_listitem.trigger_trailing_action('search_action')
        self.assertEqual(self.test_page.title,
                         'search_action action triggered')

        self.test_listitem.trigger_trailing_action('edit_action')
        self.assertEqual(self.test_page.title,
                         'edit_action action triggered')

        self.test_listitem.trigger_trailing_action('email_action')
        self.assertEqual(self.test_page.title,
                         'email_action action triggered')

    def test_trigger_nonexistent_leading_action(self):
        error = self.assertRaises(lomiriuitoolkit.ToolkitException,
                                  self.test_listitem.trigger_leading_action,
                                  ('this_action_does_not_exist'))
        self.assertEqual(str(error),
                         'The requested action not found on leading side')

    def test_trigger_nonexistent_trailing_action(self):
        error = self.assertRaises(lomiriuitoolkit.ToolkitException,
                                  self.test_listitem.trigger_trailing_action,
                                  'this_action_does_not_exist')
        self.assertEqual(str(error),
                         'The requested action not found on trailing side')

    def test_select_items(self):
        self.list_view.enable_select_mode()
        self.test_listitem.toggle_selected()
        self.assertTrue(self.test_listitem.selected)
        # select an other one
        listItem3 = self.main_view.select_single(
            'UCListItem', objectName='listitem3')
        listItem3.toggle_selected()
        self.assertTrue(listItem3.selected)

    def test_popover(self):
        self.pointing_device.click_object(self.test_listitem, button=3)
        popover = self.main_view.wait_select_single(
            'ListItemPopover', objectName='listItemContextMenu')
        self.assertRaises(
            lomiriuitoolkit._custom_proxy_objects._common.ToolkitException,
            popover.click_action_button, 'invisible_action')
        popover.click_action_button('delete_action')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus
from lomiriuitoolkit._custom_proxy_objects import _common
from lomiriuitoolkit._custom_proxy_objects import _flickable


logger = logging.getLogger(__name__)


class ActionBar(_common.LomiriUIToolkitCustomProxyObjectBase):
    """ActionBar Autopilot custom proxy object."""

    def __init__(self, *args):
        super().__init__(*args)
        # listview will only be set for a scrolling ActionBar.
        try:
            self.listview = self.select_single(objectName='actions_listview')
        except dbus.StateNotFoundError:
            self.listview = None

    def _open_overflow(self):
        """Click the overflow button and return the overflow panel"""
        actions_overflow_button = self.select_single(
            objectName='overflow_action_button')

        if not actions_overflow_button.visible:
            raise _common.ToolkitException('No actions in overflow')

        # open the popover
        self.pointing_device.click_object(actions_overflow_button)

        # the popover is not a child of the ActionBar, so use the popover
        # object to find the requested button
        try:
            popover = self.get_root_instance().select_single(
                objectName='actions_overflow_panel')
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(
                'Failed to select overflow panel')

        return popover

    def _overflow_bar_click_action_button(self, action_object_name):
        try:
            object_name = action_object_name + "_button"
            button = self.select_single(objectName=object_name)
            # In an animating header, the button is disabled until the header
            #   animation is done. Wait for that:
            button.enabled.wait_for(True)
            self.pointing_device.click_object(button)
        except dbus.StateNotFoundError:
            # the button is not in the ActionBar, but it may be in the overflow
            try:
                popover = self._open_overflow()
                popover.click_action_button(action_object_name)
            except _common.ToolkitException:
                raise _common.ToolkitException(
                    'Button not found')

    def _scrolling_bar_click_action_button(self, action_object_name):
        buttonName = action_object_name + "_button"
        try:
            self.listview.click_element(buttonName)
        except _flickable.CannotSwipeMoreToolkitException:
            # The button was found, but is not inside the ListView. This
            # happens because at the beginning and end of the ListView
            # there are list items visible inside the extra margins. But
            # the buttons are present otherwise a different error message
            # wass given.
            button = self.listview.select_single(objectName=buttonName)
            self.pointing_device.click_object(button)
        except _common.ToolkitException:
            raise _common.ToolkitException('Button not found')

    @autopilot_logging.log_action(logger.info)
    def click_action_button(self, action_object_name):
        """Click an action button of the action bar.

        :parameter object_name: The QML objectName property of the action
        :raise ToolkitException: If there is no action button with that object
            name.

        """
        if self.styleName == "ActionBarStyle":
            return self._overflow_bar_click_action_button(action_object_name)
        elif self.styleName == "ScrollingActionBarStyle":
            return self._scrolling_bar_click_action_button(action_object_name)
        else:
            raise _common.ToolkitException(
                'Unsupported style name ' + self.styleName + ' for ActionBar.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging

from lomiriuitoolkit._custom_proxy_objects import _common, _mainview

logger = logging.getLogger(__name__)


class TextField(_common.LomiriUIToolkitCustomProxyObjectBase):
    """TextField Autopilot custom proxy object."""

    def __init__(self, *args):
        super().__init__(*args)
        self.keyboard = _common.get_keyboard()

    @autopilot_logging.log_action(logger.info)
    def write(self, text, clear=True):
        """Write into the text field.

        :parameter text: The text to write.
        :parameter clear: If True, the text field will be cleared before
            writing the text. If False, the text will be appended at the end
            of the text field. Default is True.

        """
        self._ensure_focused()
        if clear:
            self.clear()
        else:
            if not self.is_empty():
                self._go_to_end()
        self.keyboard.type(text)

    @autopilot_logging.log_action(logger.info)
    def clear(self):
        """Clear the text field."""
        self._ensure_focused()
        if not self.is_empty():
            if self.hasClearButton:
                self._click_clear_button()
            else:
                self._clear_with_keys()
            self.text.wait_for('')

    def is_empty(self):
        """Return True if the text field is empty. False otherwise."""
        return self.text == ''

    @autopilot_logging.log_action(logger.debug)
    def _click_clear_button(self):
        clear_button = self.select_single(objectName='clear_button')
        if not clear_button.visible:
            self.pointing_device.click_object(self)
        self.pointing_device.click_object(clear_button)

    @autopilot_logging.log_action(logger.debug)
    def _clear_with_keys(self):
        self._select_all()
        self.keyboard.press_and_release('\b')
        if not self.is_empty():
            raise _common.ToolkitException('Failed to clear the text field.')

    @autopilot_logging.log_action(logger.debug)
    def _select_all(self):
        if not self._is_all_text_selected():
            if self._is_keyboard_osk():
                cursor = self.select_single('TextCursor', visible=True)
                self.pointing_device.click_object(
                    cursor, time_between_events=0.5)
                self.pointing_device.click_object(
                    cursor, time_between_events=0.5)
                root = self.get_root_instance()
                main_view = root.select_single(_mainview.MainView)
                popover = main_view.get_text_input_context_menu(
                    'text_input_contextmenu')
                popover.click_option_by_text('Select All')
            else:
                self._go_to_start()
                self.keyboard.press('Shift')
                self._go_to_end()
                self.keyboard.release('Shift')

    def _is_all_text_selected(self):
        return self.text == self.selectedText

    def _is_keyboard_osk(self):
        """Return True if the keyboard instance is the OSK."""
        return _common.is_maliit_process_running()

    @autopilot_logging.log_action(logger.debug)
    def _go_to_end(self):
        if self._is_keyboard_osk():
            from lomiri_keyboard.emulators.keyboard import Keyboard
            Keyboard().send_end_key()
        else:
            self.keyboard.press_and_release('End')

    @autopilot_logging.log_action(logger.debug)
    def _go_to_start(self):
        if self._is_keyboard_osk():
            from lomiri_keyboard.emulators.keyboard import Keyboard
            Keyboard().send_home_key()
        else:
            self.keyboard.press_and_release('Home')

    @autopilot_logging.log_action(logger.debug)
    def _ensure_focused(self):
        if not self.activeFocus:
            self.pointing_device.click_object(self)
            self.activeFocus.wait_for(True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
from testtools.matchers import GreaterThan

import lomiriuitoolkit
from lomiriuitoolkit import tests


class TextAreaTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_textarea.TextAreaTestCase.qml')

    def setUp(self):
        super().setUp()
        self.simple_text_area = self.main_view.select_single(
            lomiriuitoolkit.TextArea, objectName='simple_text_area')

    def test_text_area_inherits_text_field(self):
        self.assertTrue(
            issubclass(lomiriuitoolkit.TextArea, lomiriuitoolkit.TextField))

    def test_clear_with_single_line(self):
        self.simple_text_area.write('test')
        self.simple_text_area.clear()
        self.assertEqual(self.simple_text_area.text, '')

    def test_clear_with_multiple_lines(self):
        # This is a regrestion test for http://pad.lv/1359167
        self.simple_text_area.write(
            'Long text that will make it wrap into multiple lines.')
        self.assertThat(self.simple_text_area.lineCount, GreaterThan(1))
        self.simple_text_area._go_to_start()
        self.simple_text_area.clear()
        self.assertEqual(self.simple_text_area.text, '')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Header component."""

import os

from autopilot.introspection import dbus

from lomiriuitoolkit import tests


class HeaderActionsOverflowTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    tools_test_qml_file_path = os.path.join(
        dir_path, 'test_header.HeaderActionsOverflowTestCase.tools.qml')
    actions_test_qml_file_path = os.path.join(
        dir_path, 'test_header.HeaderActionsOverflowTestCase.actions.qml')

    scenarios = [
        ('deprecated tools',
            dict(test_qml_file_path=tools_test_qml_file_path)),
        ('actions',
            dict(test_qml_file_path=actions_test_qml_file_path))
    ]

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()

    def test_click_header_overflow_action_must_close_popover(self):
        """Test that clicking an action from the popover must close it.

        Regression test for http://pad.lv/1326963

        """
        self.header.click_action_button('pushStackAction')
        # the popover was created to click the action button
        # popover will be closed and destroyed immediately after
        # clicking the action button
        try:
            overflow_popover = self.main_view.select_single(
                objectName='actions_overflow_popover')
            overflow_popover.wait_until_destroyed()
        except dbus.StateNotFoundError:
            # overflow popover was already destroyed before
            # it could be selected
            pass


class HeaderContentsTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_header.HeaderContentsTestCase.qml')

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()

    def test_open_page_with_head_contents_must_change_header(self):
        """Test that setting head.contents overrides header contents."""

        label = self.header.select_single(
            'Label',
            objectName='header_title_label')
        self.assertEqual(label.visible, True)
        self.assertEqual(label.text, 'Test title')

        pushButton = self.main_view.select_single(
            'Button',
            objectName='push_button')
        self.pointing_device.move_to_object(pushButton)
        self.pointing_device.click()
        self.header.wait_for_animation()

        self.assertEqual(label.visible, False)
        headerContents = self.header.select_single(
            objectName='orange_header_contents')
        self.assertEqual(headerContents.visible, True)

    def test_close_page_with_head_contents_must_revert_header(self):
        """Test that closing a page with head.contents reverts the header."""
        pushButton = self.main_view.select_single(
            'Button',
            objectName='push_button')
        self.pointing_device.move_to_object(pushButton)
        self.pointing_device.click()

        self.header.wait_for_animation()
        headerContents = self.header.select_single(
            objectName='orange_header_contents')
        label = self.header.select_single(
            'Label',
            objectName='header_title_label')

        self.assertEqual(headerContents.visible, True)
        self.assertEqual(label.visible, False)

        self.header.click_back_button()
        self.assertEqual(headerContents.visible, False)
        self.assertEqual(label.visible, True)
        self.assertEqual(label.text, 'Test title')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import testscenarios

from testtools.matchers import Equals

from lomiriuitoolkit import emulators, lomiri_scenarios
from lomiriuitoolkit.tests import gallery


class EnabledTogglesTestCase(gallery.GalleryTestCase):

    toggles_scenarios = [
        ('checkbox unchecked', dict(
            object_name='checkbox_unchecked', initial_state=False)),
        ('checkbox checked', dict(
            object_name='checkbox_checked', initial_state=True)),
        ('switch unchecked', dict(
            object_name='switch_unchecked', initial_state=False)),
        ('switch checked', dict(
            object_name='switch_checked', initial_state=True))
    ]

    scenarios = testscenarios.multiply_scenarios(
        lomiri_scenarios.get_device_simulation_scenarios(),
        toggles_scenarios)

    def setUp(self):
        super().setUp()
        self.open_page('togglesElement')

    def test_change_toggles_state(self):
        toggle = self.main_view.select_single(
            emulators.CheckBox, objectName=self.object_name)
        self.assertThat(toggle.enabled, Equals(True))
        self.assertThat(toggle.checked, Equals(self.initial_state))

        toggle.change_state(timeout=1)
        self.assertThat(toggle.checked, Equals(not self.initial_state))


class DisabledTogglesTestCase(gallery.GalleryTestCase):

    toggles_scenarios = [
        ('checkbox disabled unchecked', dict(
            object_name='checkbox_disabled_unchecked', initial_state=False)),
        ('checkbox disabled checked', dict(
            object_name='checkbox_disabled_checked', initial_state=True)),
        ('switch disabled unchecked', dict(
            object_name='switch_disabled_unchecked', initial_state=False)),
        ('switch disabled checked', dict(
            object_name='switch_disabled_checked', initial_state=True))
    ]

    scenarios = testscenarios.multiply_scenarios(
        lomiri_scenarios.get_device_simulation_scenarios(),
        toggles_scenarios)

    def setUp(self):
        super().setUp()
        self.open_page('togglesElement')

    def test_change_toggles_state(self):
        toggle = self.main_view.select_single(
            emulators.CheckBox, objectName=self.object_name)
        self.assertThat(toggle.enabled, Equals(False))
        self.assertThat(toggle.checked, Equals(self.initial_state))

        self.assertRaises(AssertionError, toggle.change_state, timeout=1)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the QML ListItems component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class ListItemsTests(TavastiaTestCase):
    """Tests for ListItems component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"ListItemsTests")

    def test_can_select_header(self):
        """Must be able to select the Qml ListItem.Header component."""

        obj = self.app.select_single('Header')
        self.assertThat(obj, Not(Is(None)))

    def test_can_select_standard(self):
        """Must be able to select the Qml ListItem.Standard component."""

        obj = self.app.select_single('Standard',text="Standard")
        self.assertThat(obj, Not(Is(None)))
    
    def test_can_select_singlevalue(self):
        """Must be able to select the Qml ListItem.SingleValue component."""

        obj = self.app.select_single('SingleValue')
        self.assertThat(obj, Not(Is(None)))

    def test_can_select_multivalue(self):
        """Must be able to select the Qml ListItem.MultiValue component."""

        obj = self.app.select_single('MultiValue')
        self.assertThat(obj, Not(Is(None)))

    def test_can_select_subtitled(self):
        """Must be able to select the Qml ListItem.Subtitled component."""

        obj = self.app.select_single('Subtitled')
        self.assertThat(obj, Not(Is(None)))

    def test_can_select_caption(self):
        """Must be able to select the Qml ListItem.Caption component."""

        obj = self.app.select_single('Caption')
        self.assertThat(obj, Not(Is(None)))

    def test_can_select_ValueSelector(self):
        """Must be able to select the Qml ListItem.ValueSelector component."""

        obj = self.app.select_single('ValueSelector')
        self.assertThat(obj, Not(Is(None)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit units helpers."""

import logging
import os
import testscenarios
import testtools

import fixtures

from lomiriuitoolkit import units


logger = logging.getLogger(__name__)


class UnitsTestCase(testscenarios.TestWithScenarios,
                    testtools.TestCase):

    WIDTH_IN_GU = 20

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_units.UnitsTestCase.qml')

    scenarios = [
        ('with default GRID_UNIT_PX', {'grid_unit_px': '',
         'expected_pixels': 160}),
        ('with GRID_UNIT_PX environment variable set', {'grid_unit_px': '10',
         'expected_pixels': 200})
    ]

    def setUp(self):
        self.useFixture(fixtures.EnvironmentVariable(
            'GRID_UNIT_PX', self.grid_unit_px))
        super().setUp()

    def test_gu(self):
        pixels = units.gu(self.WIDTH_IN_GU)
        self.assertEquals(pixels, self.expected_pixels)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from lomiriuitoolkit._custom_proxy_objects import _textfield


class TextArea(_textfield.TextField):
    """TextArea autopilot emulator."""

    def clear(self):
        """Clear the text area."""
        self._ensure_focused()
        if not self.is_empty():
            self._clear_with_keys()
            self.text.wait_for('')

    def _go_to_end(self):
        # We override this because the text areas can have more than one line.
        if self._is_keyboard_osk():
            from lomiri_keyboard.emulators.keyboard import Keyboard
            osk = Keyboard()
            while self.cursorPosition != len(self.text):
                # Move to the end of the line below
                osk.send_down_key()
                osk.send_end_key()
        else:
            self.keyboard.press_and_release('Ctrl+End')

    def _go_to_start(self):
        # We override this because the text areas can have more than one line.
        if self._is_keyboard_osk():
            from lomiri_keyboard.emulators.keyboard import Keyboard
            osk = Keyboard()
            while self.cursorPosition != 0:
                # Move to the start of the line above
                osk.send_up_key()
                osk.send_home_key()
        else:
            self.keyboard.press_and_release('Ctrl+Home')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from lomiriuitoolkit._custom_proxy_objects import _common


class Tabs(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Tabs Autopilot custom proxy object."""

    def get_current_tab(self):
        """Return the currently selected tab."""
        return self._get_tab(self.selectedTabIndex)

    def _get_tab(self, index):
        tabs = self._get_tabs()
        for tab in tabs:
            if tab.index == index:
                return tab
        else:
            raise _common.ToolkitException(
                'There is no tab with index {0}.'.format(index))

    def _get_tabs(self):
        return self.select_many('Tab')

    def get_number_of_tabs(self):
        """Return the number of tabs."""
        return len(self._get_tabs())



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Header component."""

import os
import testtools
from time import sleep

from autopilot.input._common import get_center_point
from autopilot import platform

from lomiriuitoolkit import tests


class CaretTextInputTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    textfield_qml_file_path = os.path.join(
        dir_path, 'test_textinput.textfield.qml')
    textarea_qml_file_path = os.path.join(
        dir_path, 'test_textinput.textarea.qml')
    customfield_qml_file_path = os.path.join(
        dir_path, 'test_textinput.textfield_custom.qml')
    header_qml_file_path = os.path.join(
        dir_path, 'test_textinput.header.qml')

    scenarios = [
        ('textfield',
            dict(test_qml_file_path=textfield_qml_file_path,
                 objectName='textfield')),
        ('textarea',
            dict(test_qml_file_path=textarea_qml_file_path,
                 objectName='textarea')),
        ('customfield',
            dict(test_qml_file_path=customfield_qml_file_path,
                 objectName='textfield')),
        ('header',
            dict(test_qml_file_path=header_qml_file_path,
                 objectName='textfield')),
    ]

    def get_command_line(self, command_line):
        command_line.append('-touch')
        return command_line

    def setUp(self):
        super().setUp()
        self.textfield = self.main_view.select_single(
            objectName=self.objectName)
        self.assertFalse(self.textfield.focus)

    def select_cursor(self, positionProperty):
        # The cursor may not receive events right away
        sleep(1)
        return self.main_view.select_single(
            objectName=positionProperty + '_draggeditem')

    def test_caret_hidden_if_empty(self):
        cursorName = 'text_cursor_style_caret_cursorPosition'
        self._assert_not_visible(objectName=cursorName)
        self.pointing_device.click_object(self.textfield)
        self.assertTrue(self.textfield.focus)
        self._assert_not_visible(objectName=cursorName)

    def test_caret_visible_on_focus(self):
        cursorName = 'text_cursor_style_caret_cursorPosition'
        self._assert_not_visible(objectName=cursorName)
        self.textfield.write('Lorem ipsum')
        self.pointing_device.click_object(self.textfield)
        self.main_view.select_single(objectName=cursorName)

    def test_caret_hide_while_typing(self):
        self.pointing_device.click_object(self.textfield)
        self.assertTrue(self.textfield.focus)
        cursor = self.select_cursor('cursorPosition')

        self.textfield.write('Lorem ipsum')
        self.assertFalse(cursor.visible)

    def test_caret_visible_after_tapping(self):
        self.test_caret_hide_while_typing()
        self.pointing_device.click_object(self.textfield)
        self.select_cursor('cursorPosition')

    @testtools.skipUnless(platform.model() == 'Desktop',
                          'Desktop only due to non-OSK key used.')
    def test_caret_visible_after_selecting(self):
        self.test_caret_hide_while_typing()
        # Select a character
        self.keyboard.press_and_release('Shift+Left')
        self.select_cursor('selectionEnd')


class InsertModeTextInputTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    textfield_qml_file_path = os.path.join(
        dir_path, 'test_textinput.textfield.qml')
    textarea_qml_file_path = os.path.join(
        dir_path, 'test_textinput.textarea.qml')
    customfield_qml_file_path = os.path.join(
        dir_path, 'test_textinput.textfield_custom.qml')
    header_qml_file_path = os.path.join(
        dir_path, 'test_textinput.header.qml')

    scenarios = [
        ('textfield',
         dict(test_qml_file_path=textfield_qml_file_path,
              objectName='textfield')),
        ('textarea',
         dict(test_qml_file_path=textarea_qml_file_path,
              objectName='textarea')),
        ('customfield',
         dict(test_qml_file_path=customfield_qml_file_path,
              objectName='textfield')),
        ('header',
         dict(test_qml_file_path=header_qml_file_path,
              objectName='textfield')),
    ]

    def get_command_line(self, command_line):
        command_line.append('-touch')
        return command_line

    def setUp(self):
        super().setUp()
        self.textfield = self.main_view.select_single(
            objectName=self.objectName)
        self.assertFalse(self.textfield.focus)

    def assert_buttons(self, texts):
        popover = self.main_view.get_text_input_context_menu(
            'text_input_contextmenu')
        for text in texts:
            button = popover._get_button(text)
            self.assertTrue(button.visible)

    def assert_discard_popover(self):
        # Discard popover by tap
        self.pointing_device.move(
            self.textfield.globalRect.x + self.textfield.width * 0.6,
            self.textfield.globalRect.y + self.textfield.height * 0.95)
        self.pointing_device.click()

        self._assert_not_visible(objectName='text_input_contextmenu')

    def select_cursor(self, positionProperty):
        # The cursor may not receive events right away
        sleep(1)
        return self.main_view.select_single(
            objectName=positionProperty + '_draggeditem')

    def test_popover_not_obscured(self):
        self.textfield.write('Lorem ipsum')
        self.pointing_device.click_object(self.textfield)
        cursor = self.select_cursor('cursorPosition')
        self.pointing_device.click_object(cursor)
        popover = self.main_view.get_text_input_context_menu(
            'text_input_contextmenu')
        self.assertTrue(popover.globalRect.y > 0,
                        '%s <= 0' % popover.globalRect.y)

    def test_header_undisturbed_by_text_handlers(self):
        # Verify that handlers aren't accidentally placed at absolute 0/0
        self.pointing_device.click_object(self.textfield)
        # Back will hide when pressed
        back = self.main_view.select_single(objectName='customBackButton')
        self.main_view.get_header().click_custom_back_button()
        self.assertFalse(back.visible)

    def test_popover_visible_after_tapping_caret(self):
        # Insert Mode
        self.textfield.write('Lorem ipsum')
        self.pointing_device.click_object(self.textfield)
        cursor = self.select_cursor('cursorPosition')
        self.pointing_device.click_object(cursor)
        self.assert_buttons(['Select All', 'Paste'])
        self.assert_discard_popover()

    def test_popover_visible_after_dragging_caret(self):
        # Insert Mode
        self.textfield.write('Lorem ipsum')
        self.pointing_device.click_object(self.textfield)
        cursor = self.select_cursor('cursorPosition')
        x, y = get_center_point(cursor)
        self.pointing_device.drag(x, y, x + self.textfield.width // 2, y)
        self.assert_buttons(['Select All', 'Paste'])
        self.assert_discard_popover()

    @testtools.skipIf(platform.model() == 'Desktop', 'Touch only')
    def test_popover_visible_after_long_press(self):
        # Select Mode
        self.textfield.write('Lorem ipsum')
        self.pointing_device.move(
            self.textfield.globalRect.x + self.textfield.width // 8,
            self.textfield.globalRect.y + self.textfield.height // 2)
        # Long press to select a word
        # FIXME: input.Mouse doesn't support long press
        # press_duration doesn't work here
        # self.pointing_device.click(press_duration=2.0)
        self.pointing_device.press()
        sleep(2)
        self.pointing_device.release()
        self.assert_buttons(['Cut', 'Copy', 'Paste'])
        self.assert_discard_popover()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import lomiriuitoolkit
from lomiriuitoolkit import tests


class OptionSelectorCustomDelegateTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path,
        'test_optionselector.OptionSelectorCustomDelegateTestCase.qml')

    def setUp(self):
        super().setUp()
        self.option_selector = self.main_view.select_single(
            lomiriuitoolkit.OptionSelector,
            objectName="test_option_selector_collapsed")

    def test_get_option_count(self):
        """get_option_count() must return number of options"""
        self.assertEqual(self.option_selector.get_option_count(), 5)

    def test_expand(self):
        """expand() must expand a collapsed option elector"""
        self.assertFalse(self.option_selector.currentlyExpanded)
        self.option_selector._expand()
        self.assertTrue(self.option_selector.currentlyExpanded)

    def test_negative_select_text(self):
        """select_text() must raise a ValueError if text does not exist"""
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            lambda: self.option_selector.select_option(text="this should fail")
        )
        self.assertEqual(
            "OptionSelectorDelegate with kwargs {'text': 'this should fail'}"
            " not found",
            str(error)
        )

    def test_select_option(self):
        """select_text() must select the text in the OptionSelector"""
        self.option_selector.select_option(objectName="three")
        self.assertEqual(2, self.option_selector.selectedIndex)

    def test_get_selected_text(self):
        """get_selected_text() must return the text selected item"""
        self.option_selector.select_option(text="Value 2")
        self.assertEqual(1, self.option_selector.selectedIndex)
        self.assertEqual(
            self.option_selector.get_selected_text(), "Value 2")

    def test_same_item_2_times(self):
        """Emulator must be able to select 2 items in a row"""
        self.option_selector.select_option(text="Value 3")
        self.assertEqual(
            self.option_selector.get_selected_text(), "Value 3")
        self.assertEqual(2, self.option_selector.selectedIndex)
        self.option_selector.select_option(text="Value 3")
        self.assertEqual(
            self.option_selector.get_selected_text(), "Value 3")
        self.assertEqual(2, self.option_selector.selectedIndex)


class OptionSelectorTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_optionselector.OptionSelectorTestCase.qml')

    def setUp(self):
        super().setUp()
        self.option_selector = self.main_view.select_single(
            lomiriuitoolkit.OptionSelector,
            objectName="option_selector")

    def test_get_option_count(self):
        """get_option_count() must return number of options"""
        self.assertEqual(self.option_selector.get_option_count(), 5)

    def test_negative_select_text(self):
        """select_text() must raise a ValueError if object and
        text does not exist"""
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            lambda: self.option_selector.select_option('Label',
                                                       text="this should fail")
        )
        self.assertEqual(
            "OptionSelectorDelegate with args ('Label',) and kwargs {'text': "
            "'this should fail'} not found",
            str(error)
        )

    def test_select_option(self):
        """select_text() must select the text in the OptionSelector"""
        self.option_selector.select_option('UCLabel', text="Green")
        self.assertEqual(2, self.option_selector.selectedIndex)

    def test_get_selected_text(self):
        """get_selected_text() must return the text selected item"""
        self.option_selector.select_option('UCLabel', text="Blue")
        self.assertEqual(1, self.option_selector.selectedIndex)
        self.assertEqual(
            self.option_selector.get_selected_text(), "Blue")

    def test_same_item_2_times(self):
        """Emulator must be able to select 2 items in a row"""
        self.option_selector.select_option('UCLabel', text="Green")
        self.assertEqual(
            self.option_selector.get_selected_text(), "Green")
        self.assertEqual(2, self.option_selector.selectedIndex)
        self.option_selector.select_option('UCLabel', text="Green")
        self.assertEqual(
            self.option_selector.get_selected_text(), "Green")
        self.assertEqual(2, self.option_selector.selectedIndex)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os

from lomiriuitoolkit import tests


class DialogTestCase(tests.QMLFileAppTestCase):
    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_dialog.DialogTestCase.qml')

    def test_dialog(self):
        open_button = self.main_view.wait_select_single('Button',
                                                        objectName='button')
        self.pointing_device.click_object(open_button)
        dialog = self.main_view.wait_select_single('Dialog',
                                                   title="Dialog")
        text_area = dialog.wait_select_single('TextArea',
                                              objectName='textfield_standard')
        text_area.write('test')
        self.assertEqual(text_area.text, 'test')
        lower_button = dialog.wait_select_single('Button',
                                                 text='Lower button')
        self.pointing_device.click_object(lower_button)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import copy
import json
import logging
import os
import shutil
import subprocess
import tempfile

import fixtures
from autopilot import display
from gi.repository import Gio

from lomiriuitoolkit import base


DEFAULT_QML_FILE_CONTENTS = ("""
import QtQuick 2.0
import Lomiri.Components 1.1

MainView {
    width: units.gu(48)
    height: units.gu(60)

    Label {
        objectName: 'testLabel'
        text: 'Test application.'
    }
}
""")
DEFAULT_DESKTOP_FILE_DICT = {
    'Type': 'Application',
    'Name': 'test',
    'Exec': '{launcher} {qml_file_path}',
    'Icon': 'Not important'
}


logger = logging.getLogger(__name__)


class FakeApplication(fixtures.Fixture):

    def __init__(
            self, qml_file_contents=DEFAULT_QML_FILE_CONTENTS,
            desktop_file_dict=None, url_dispatcher_protocols=None):
        super().__init__()
        self._qml_file_contents = qml_file_contents
        if desktop_file_dict is None:
            self._desktop_file_dict = copy.deepcopy(DEFAULT_DESKTOP_FILE_DICT)
        else:
            self._desktop_file_dict = copy.deepcopy(desktop_file_dict)
        self.url_dispatcher_protocols = url_dispatcher_protocols

    def setUp(self):
        super().setUp()
        self.qml_file_path, self.desktop_file_path = (
            self._create_test_application())
        desktop_file_name = os.path.basename(self.desktop_file_path)
        self.application_name, _ = os.path.splitext(desktop_file_name)
        if self.url_dispatcher_protocols:
            self.url_dispatcher_file_path = (
                self._register_url_dispatcher_protocols(self.application_name))

    def _create_test_application(self):
        qml_file_path = self._write_test_qml_file()
        self.addCleanup(os.remove, qml_file_path)
        desktop_file_path = self._write_test_desktop_file(qml_file_path)
        self.addCleanup(os.remove, desktop_file_path)
        return qml_file_path, desktop_file_path

    def _write_test_qml_file(self):
        qml_file = self._named_temporary_file(suffix='.qml')
        qml_file.write(self._qml_file_contents)
        qml_file.close()
        return qml_file.name

    def _named_temporary_file(self, dir=None, mode='w+t',
                              delete=False, suffix=''):
        # Discard files with underscores which look like an APP_ID to Unity
        # See https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1329141
        chars = tempfile._RandomNameSequence.characters.strip("_")
        tempfile._RandomNameSequence.characters = chars
        return tempfile.NamedTemporaryFile(dir=dir, mode=mode,
                                           delete=delete, suffix=suffix)

    def _write_test_desktop_file(self, qml_file_path):
        desktop_file_dir = self._get_local_desktop_file_directory()
        if not os.path.exists(desktop_file_dir):
            os.makedirs(desktop_file_dir)
        desktop_file = self._named_temporary_file(suffix='.desktop',
                                                  dir=desktop_file_dir)
        self._desktop_file_dict['Exec'] = (
            self._desktop_file_dict['Exec'].format(
                launcher=base.get_toolkit_launcher_command(),
                qml_file_path=qml_file_path))
        desktop_file.write('[Desktop Entry]\n')
        for key, value in self._desktop_file_dict.items():
            desktop_file.write('{key}={value}\n'.format(key=key, value=value))
        desktop_file.close()
        return desktop_file.name

    def _get_local_desktop_file_directory(self):
        return os.path.join(
            os.environ.get('HOME'), '.local', 'share', 'applications')

    def _register_url_dispatcher_protocols(self, application_name):
        url_dispatcher_file_path = self._write_url_dispatcher_file(
            application_name)
        self.addCleanup(os.remove, url_dispatcher_file_path)
        self._update_url_dispatcher_directory(url_dispatcher_file_path)
        return url_dispatcher_file_path

    def _write_url_dispatcher_file(self, application_name):
        url_dispatcher_dir = self._get_local_url_dispatcher_directory()
        if not os.path.exists(url_dispatcher_dir):
            os.makedirs(url_dispatcher_dir)

        protocol_list = [
            {'protocol': protocol}
            for protocol in self.url_dispatcher_protocols]

        url_dispatcher_file_path = os.path.join(
            url_dispatcher_dir, application_name + '.url-dispatcher')
        with open(url_dispatcher_file_path, 'w') as url_dispatcher_file:
            url_dispatcher_file.write(json.dumps(protocol_list))

        return url_dispatcher_file_path

    def _get_local_url_dispatcher_directory(self):
        return os.path.join(
            os.environ.get('HOME'), '.config', 'url-dispatcher', 'urls')

    def _update_url_dispatcher_directory(self, url_dispatcher_file_path):
        # FIXME This should be updated calling
        # initctl start url-dispatcher-update-user, but it is not working.
        # https://bugs.launchpad.net/lomiri/+source/url-dispatcher/+bug/1461496
        # --elopio - 2015-06-02
        subprocess.check_output(
            '/usr/lib/*/url-dispatcher/update-directory ' +
            url_dispatcher_file_path, shell=True)


class FakeHome(fixtures.Fixture):

    # We copy the Xauthority file to allow executions using XVFB. If it is not
    # on the user's HOME directory, nothing will happen.
    should_copy_xauthority_file = True

    def __init__(self, directory=None):
        super().__init__()
        self.directory = directory

    def setUp(self):
        super().setUp()
        self.directory = self._make_directory_if_not_specified()
        if self.should_copy_xauthority_file:
            self._copy_xauthority_file(self.directory)
        self.useFixture(
            fixtures.EnvironmentVariable('HOME', newvalue=self.directory))

    def _make_directory_if_not_specified(self):
        if self.directory is None:
            parent_directory = os.path.join(
                os.environ.get('HOME'), 'autopilot', 'fakeenv')
            if not os.path.exists(parent_directory):
                os.makedirs(parent_directory)
            temp_dir_fixture = fixtures.TempDir(parent_directory)
            self.useFixture(temp_dir_fixture)
            return temp_dir_fixture.path
        else:
            return self.directory

    def _copy_xauthority_file(self, directory):
        """Copy the .Xauthority file if it exists in the user's home."""
        xauthority_file_path = os.path.join(
            os.environ.get('HOME'), '.Xauthority')
        if os.path.isfile(xauthority_file_path):
            shutil.copyfile(
                xauthority_file_path,
                os.path.join(directory, '.Xauthority'))


class HideUnity7Launcher(fixtures.Fixture):

    """Hide the Unity7 launcher if it is visible and restore it on clean up."""

    _UNITYSHELL_GSETTINGS_SCHEMA = 'org.compiz.unityshell'
    _UNITYSHELL_GSETTINGS_PATH = (
        '/org/compiz/profiles/unity/plugins/unityshell/')
    _UNITYSHELL_LAUNCHER_KEY = 'launcher-hide-mode'
    _UNITYSHELL_LAUNCHER_HIDDEN_MODE = 1  # launcher hidden

    def setUp(self):
        super().setUp()
        self._hide_launcher()

    def _hide_launcher(self):
        if (self._UNITYSHELL_GSETTINGS_SCHEMA in
                Gio.Settings.list_relocatable_schemas()):
            unityshell_schema = Gio.Settings.new_with_path(
                self._UNITYSHELL_GSETTINGS_SCHEMA,
                self._UNITYSHELL_GSETTINGS_PATH)
            self._hide_launcher_from_schema(unityshell_schema)
        else:
            logger.warning('Unity Shell gsettings schema not found.')

    def _hide_launcher_from_schema(self, unityshell_schema):
        original_mode = self._get_launcher_mode(unityshell_schema)
        self.addCleanup(
            self._set_launcher_mode, unityshell_schema, original_mode)
        self._set_launcher_mode(
            unityshell_schema, self._UNITYSHELL_LAUNCHER_HIDDEN_MODE)

    def _get_launcher_mode(self, unityshell_schema):
        return unityshell_schema.get_int(self._UNITYSHELL_LAUNCHER_KEY)

    def _set_launcher_mode(self, unityshell_schema, mode):
        unityshell_schema.set_int(self._UNITYSHELL_LAUNCHER_KEY, mode)


class SimulateDevice(fixtures.Fixture):

    def __init__(self, app_width, app_height, grid_unit_px):
        super().__init__()
        self.app_width = app_width
        self.app_height = app_height
        self.grid_unit_px = grid_unit_px
        self._screen = None

    def setUp(self):
        super().setUp()
        if self._is_geometry_larger_than_display(
                self.app_width, self.app_height):
            scale_divisor = self._get_scale_divisor()
            self.grid_unit_px = self.grid_unit_px // scale_divisor
            self.app_width = self.app_width // scale_divisor
            self.app_height = self.app_height // scale_divisor
        self.useFixture(
            fixtures.EnvironmentVariable(
                'GRID_UNIT_PX', str(self.grid_unit_px)))

    def _get_scale_divisor(self):
        scale_divisor = 2
        while self._is_geometry_larger_than_display(
                self.app_width // scale_divisor,
                self.app_height // scale_divisor):
            scale_divisor = scale_divisor * 2
        return scale_divisor

    def _is_geometry_larger_than_display(self, width, height):
        screen = self._get_screen()
        screen_width = screen.get_screen_width()
        screen_height = screen.get_screen_height()
        return (width > screen_width) or (height > screen_height)

    def _get_screen(self):
        if self._screen is None:
            self._screen = display.Display.create()
        return self._screen



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Common helpers for Lomiri UI Toolkit Autopilot custom proxy objects."""

import logging
import os
import subprocess
import time
from distutils import version
from gi.repository import Gio

import autopilot
from autopilot import (
    input,
    logging as autopilot_logging,
    platform
)
from autopilot.introspection import dbus

logger = logging.getLogger(__name__)

MALIIT = 'maliit-server'


class ToolkitException(Exception):
    """Exception raised when there is an error with the custom proxy object."""


def get_pointing_device():
    """Return the pointing device depending on the platform.

    If the platform is `Desktop`, the pointing device will be a `Mouse`.
    If not, the pointing device will be `Touch`.

    """
    if platform.model() == 'Desktop':
        input_device_class = input.Mouse
    else:
        input_device_class = input.Touch
    return input.Pointer(device=input_device_class.create())


def get_keyboard():
    """Return the keyboard device."""
    if is_maliit_process_running():
        configure_osk_settings()
        restart_maliit_with_testability()
        return input.Keyboard.create('OSK')
    else:
        return input.Keyboard.create()


def restart_maliit_with_testability():
    """Restart maliit-server with testability enabled."""
    pid = get_process_pid(MALIIT)
    if _is_testability_enabled_for_process(pid):
        return
    _stop_process(MALIIT)
    _start_process(MALIIT, 'QT_LOAD_TESTABILITY=1')
    # This is needed to work around https://launchpad.net/bugs/1248913
    time.sleep(5)


def configure_osk_settings():
    """Configure OSK ready for testing by turning off all helpers."""
    gsettings = Gio.Settings.new("com.lomiri.keyboard.maliit")
    gsettings.set_string("active-language", "en")
    gsettings.set_boolean("auto-capitalization", False)
    gsettings.set_boolean("auto-completion", False)
    gsettings.set_boolean("predictive-text", False)
    gsettings.set_boolean("spell-checking", False)
    gsettings.set_boolean("double-space-full-stop", False)


def _is_testability_enabled_for_process(pid):
    """Return True if testability is enabled for specified process."""
    proc_env = '/proc/{pid}/environ'.format(pid=pid)
    command = ['cat', proc_env]
    try:
        output = subprocess.check_output(
            command,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        )
    except subprocess.CalledProcessError as e:
        e.args += ('Failed to get environment for pid {}: {}.'.format(
            pid, e.output),)
        raise
    return output.find('QT_LOAD_TESTABILITY=1') >= 0


def _stop_process(proc_name):
    """Stop process with name proc_name."""
    logger.info('Stoping process {}.'.format(proc_name))
    command = ['/sbin/initctl', 'stop', proc_name]
    try:
        output = subprocess.check_output(
            command,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        )
        logger.info(output)
    except subprocess.CalledProcessError as e:
        e.args += ('Failed to stop {}: {}.'.format(proc_name, e.output),)
        raise


def _start_process(proc_name, *args):
    """Start a process.

    :param proc_name: The name of the process.
    :param args: The arguments to be used when starting the job.
    :return: The process id of the started job.
    :raises CalledProcessError: if the job failed to start.

    """
    logger.info('Starting job {} with arguments {}.'.format(proc_name, args))
    command = ['/sbin/initctl', 'start', proc_name] + list(args)
    try:
        output = subprocess.check_output(
            command,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        )
        logger.info(output)
        pid = get_process_pid(proc_name)
    except subprocess.CalledProcessError as e:
        e.args += ('Failed to start {}: {}.'.format(proc_name, e.output),)
        raise
    else:
        return pid


def get_process_pid(proc_name):
    """Return the process id of a running job.

    :param str name: The name of the job.
    :raises ToolkitException: if the job is not running.

    """
    status = get_process_status(proc_name)
    if "start/" not in status:
        raise ToolkitException(
            '{} is not in the running state.'.format(proc_name))
    return int(status.split()[-1])


def get_process_status(name):
    """
    Return the status of a process.

    :param str name: The name of the process.
    :return: Status of process as string, or zero length string if process
             is not found.

    """
    try:
        return subprocess.check_output([
            '/sbin/initctl',
            'status',
            name
        ], universal_newlines=True, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        return ''


def is_process_running(name):
    """Return True if the process is running. Otherwise, False.

    :param str name: The name of the process.
    :raises ToolkitException: if not possible to get status of the process.

    """
    return 'start/' in get_process_status(name)


def is_maliit_process_running():
    """Return True if malitt-server process is running, False otherwise."""
    # FIXME: lp#1542224 Use Maliit by default
    if 'UITK_USE_MALIIT' not in os.environ:
        logger.info("Not going to use Maliit\
            - set UITK_USE_MALIIT to enable it")
        return False
    if is_process_running(MALIIT):
        logger.info('Using Maliit for keyboard input')
        return True
    logger.info('Maliit enabled but not running on the system')
    return False


def check_autopilot_version():
    """Check that the Autopilot installed version matches the one required.

    :raise ToolkitException: If the installed Autopilot version does't
        match the required by the custom proxy objects.

    """
    installed_version = version.LooseVersion(autopilot.version)
    if installed_version < version.LooseVersion('1.4'):
        raise ToolkitException(
            'The custom proxy objects require Autopilot 1.4 or higher.')


class LomiriUIToolkitCustomProxyObjectBase(dbus.CustomEmulatorBase):
    """A base class for all the Lomiri UI Toolkit custom proxy objects."""

    def __init__(self, *args):
        check_autopilot_version()
        super().__init__(*args)
        self.pointing_device = get_pointing_device()

    def is_flickable(self):
        """Check if the object is flickable.

        If the object has a flicking attribute, we consider it as a flickable.

        :return: True if the object is flickable. False otherwise.

        """
        try:
            self.flicking
            return True
        except AttributeError:
            return False

    @autopilot_logging.log_action(logger.info)
    def swipe_into_view(self):
        """Make the object visible.

        Currently it works only when the object needs to be swiped vertically.
        TODO implement horizontal swiping. --elopio - 2014-03-21

        """
        flickable_parent = self._get_flickable_parent()
        flickable_parent.swipe_child_into_view(self)

    def _get_flickable_parent(self):
        parent = self.get_parent()
        root = self.get_root_instance()
        while parent.id != root.id:
            if parent.is_flickable():
                return parent
            parent = parent.get_parent()
        raise ToolkitException(
            "The element is not contained in a Flickable so it can't be "
            "swiped into view.")

    def _get_top_container(self):
        """Return the top-most container with a globalRect."""
        root = self.get_root_instance()
        parent = self.get_parent()
        top_container = None
        while parent.id != root.id:
            if hasattr(parent, 'globalRect'):
                top_container = parent

            parent = parent.get_parent()

        if top_container is None:
            raise ToolkitException('Could not find the top-most container.')
        else:
            return top_container



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Lomiri UI Toolkit Autopilot custom proxy objects."""

from lomiriuitoolkit._custom_proxy_objects._actionbar import ActionBar
from lomiriuitoolkit._custom_proxy_objects._sections import Sections
from lomiriuitoolkit._custom_proxy_objects._checkbox import CheckBox
from lomiriuitoolkit._custom_proxy_objects._common import (
    check_autopilot_version,
    get_keyboard,
    get_pointing_device,
    ToolkitException,
    LomiriUIToolkitCustomProxyObjectBase,
)
from lomiriuitoolkit._custom_proxy_objects._flickable import QQuickFlickable
from lomiriuitoolkit._custom_proxy_objects._header import (
    AppHeader,
    Header,
)

from lomiriuitoolkit._custom_proxy_objects.popups import (
    Dialog,
    ActionSelectionPopover,
)

from lomiriuitoolkit._custom_proxy_objects import listitems
from lomiriuitoolkit._custom_proxy_objects._uclistitem import (
    UCListItem
)
from lomiriuitoolkit._custom_proxy_objects._mainview import MainView
from lomiriuitoolkit._custom_proxy_objects._optionselector import (
    OptionSelector
)
from lomiriuitoolkit._custom_proxy_objects import pickers
from lomiriuitoolkit._custom_proxy_objects import popups
from lomiriuitoolkit._custom_proxy_objects._qquickgridview import (
    QQuickGridView
)
from lomiriuitoolkit._custom_proxy_objects._qquicklistview import (
    QQuickListView
)
from lomiriuitoolkit._custom_proxy_objects._tabbar import TabBar
from lomiriuitoolkit._custom_proxy_objects._tabs import Tabs
from lomiriuitoolkit._custom_proxy_objects._textarea import TextArea
from lomiriuitoolkit._custom_proxy_objects._textfield import TextField
from lomiriuitoolkit._custom_proxy_objects._toolbar import Toolbar
from lomiriuitoolkit._custom_proxy_objects._lomirilistview import (
    LomiriListView11,
    LomiriListView,
)

__all__ = [
    'AppHeader',
    'ActionBar',
    'ActionSelectionPopover',
    'check_autopilot_version',
    'CheckBox',
    'get_keyboard',
    'get_pointing_device',
    'Header',
    'Dialog',
    'listitems',
    'UCListItem',
    'MainView',
    'OptionSelector',
    'pickers',
    'popups',
    'QQuickFlickable',
    'QQuickGridView',
    'QQuickListView',
    'Sections',
    'TabBar',
    'Tabs',
    'TextArea',
    'TextField',
    'Toolbar',
    'ToolkitException',
    'LomiriListView11',
    'LomiriListView',
    'LomiriUIToolkitCustomProxyObjectBase',
]



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus

from lomiriuitoolkit._custom_proxy_objects import (
    _common,
    _tabbar
)


_NO_TABS_ERROR = 'The MainView has no Tabs.'


logger = logging.getLogger(__name__)


class AppHeader(_common.LomiriUIToolkitCustomProxyObjectBase):
    """AppHeader Autopilot custom proxy object."""

    def __init__(self, *args):
        super().__init__(*args)
        if not hasattr(self, 'useDeprecatedToolbar'):
            self.useDeprecatedToolbar = None

    def ensure_visible(self):
        if not self._is_visible():
            self._show()

    def _is_visible(self):
        return self.y == 0

    def _show(self):
        # FIXME This will fail if the header is not linked to a flickable that
        # fills the main view. The header has a flickable property but it
        # can't be read by autopilot. See bug http://pad.lv/1318829
        top_container = self._get_top_container()
        # Make the drag range be a multiple of the drag "rate" value.
        # Workarounds https://bugs.launchpad.net/mir/+bug/1399690
        rate = 10
        start_x = stop_x = self.globalRect.x + self.globalRect.width // 2
        start_y = top_container.globalRect.y + 5
        stop_y = start_y + self.globalRect.height // rate * rate
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y, rate)
        self.y.wait_for(0)

    def wait_for_animation(self):
        try:
            style = self.select_single(objectName='PageHeadStyle')
            style.animating.wait_for(False)
        except dbus.StateNotFoundError:
            # AppHeader is not using the new PageHeadStyle,
            # so no need to wait.
            return

        # Wait showing/hiding animation of the header.
        self.moving.wait_for(False)

    @autopilot_logging.log_action(logger.info)
    def switch_to_section_by_index(self, index):
        """Select a section in the header divider

        :parameter index: The index of the section to select
        :raise ToolkitEmulatorException: If the selection index is out of
                range or useDeprecatedToolbar is set.

        """
        self.ensure_visible()

        if self.useDeprecatedToolbar:
            raise _common.ToolkitException('Old header has no sections')

        self.wait_for_animation()
        try:
            # Lomiri.Components >=1.3
            sections = self.select_single(
                'Sections', objectName='headerSectionsItem')
            sections.click_section_button(index)
        except dbus.StateNotFoundError:
            # Lomiri.Components < 1.3, has no headerSectionsItem.
            try:
                object_name = "section_button_" + str(index)
                button = self.select_single(objectName=object_name)
            except dbus.StateNotFoundError:
                raise _common.ToolkitException(
                    'Button for section with given index not found')

            self.pointing_device.click_object(button)

    def get_selected_section_index(self):
        if self.useDeprecatedToolbar:
            raise _common.ToolkitException('Old header has no sections')

        self.wait_for_animation()
        try:
            # Lomiri.Components >=1.3
            sections = self.select_single(
                'Sections', objectName='headerSectionsItem')
            return sections.selectedIndex
        except dbus.StateNotFoundError:
            # Lomiri.Components < 1.3, has no headerSectionsItem.
            sectionsProperties = self.select_single(
                'QQuickItem', objectName='sectionsProperties')
            return sectionsProperties.selectedIndex

    def click_back_button(self):
        self.ensure_visible()

        if self.useDeprecatedToolbar:
            raise _common.ToolkitException('Old header has no back button')
        try:
            self.wait_for_animation()
            back_button = self.select_single(objectName='backButton')
        except dbus.StateNotFoundError:
            raise _common.ToolkitException('Missing back button in header')
        if not back_button.visible:
            raise _common.ToolkitException('Back button in header not visible')
        self.pointing_device.click_object(back_button)
        self.wait_for_animation()

    def click_custom_back_button(self):
        self.ensure_visible()

        if self.useDeprecatedToolbar:
            raise _common.ToolkitException(
                'Old header has no custom back button')
        try:
            self.wait_for_animation()
            custom_back_button = self.select_single(
                objectName='customBackButton')
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(
                'Missing custom back button in header')
        if not custom_back_button.visible:
            raise _common.ToolkitException(
                'Custom back button in header not visible')
        self.pointing_device.click_object(custom_back_button)
        self.wait_for_animation()

    def _get_animating(self):
        if self.useDeprecatedToolbar:
            tab_bar_style = self.select_single('TabBarStyle')
            return tab_bar_style.animating
        else:
            return False

    @autopilot_logging.log_action(logger.info)
    def switch_to_next_tab(self):
        """Open the next tab.

        :raise ToolkitException: If the main view has no tabs.

        """
        self.ensure_visible()

        if self.useDeprecatedToolbar:
            self._switch_to_next_tab_in_deprecated_tabbar()
        else:
            self._switch_to_next_tab_in_drawer()

    def _switch_to_next_tab_in_deprecated_tabbar(self):
        try:
            tab_bar = self.select_single(_tabbar.TabBar)
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(_NO_TABS_ERROR)
        tab_bar.switch_to_next_tab()
        self._get_animating().wait_for(False)

    def _switch_to_next_tab_in_drawer(self):
        self.wait_for_animation()
        tabs_model_properties = self.select_single(
            'QQuickItem', objectName='tabsModelProperties')
        if tabs_model_properties.count == 0:
            raise _common.ToolkitException(_NO_TABS_ERROR)
        next_tab_index =\
            (tabs_model_properties.selectedIndex + 1)\
            % tabs_model_properties.count
        self._switch_to_tab_in_drawer_by_index(next_tab_index)

    @autopilot_logging.log_action(logger.info)
    def switch_to_tab_by_index(self, index):
        """Open a tab. This only supports the new tabs in the header

        :parameter index: The index of the tab to open.
        :raise ToolkitException: If the tab index is out
                of range or useDeprecatedToolbar is set.

        """
        self.ensure_visible()

        if self.useDeprecatedToolbar:
            raise _common.ToolkitException(
                "AppHeader.switch_to_tab_by_index only works with new header")
        else:
            self._switch_to_tab_in_drawer_by_index(index)

    def _switch_to_tab_in_drawer_by_index(self, index):
        self.wait_for_animation()
        try:
            tabs_drawer_button = self.select_single(objectName='tabsButton')
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(_NO_TABS_ERROR)
        self.pointing_device.click_object(tabs_drawer_button)

        tabs_model_properties = self.select_single(
            'QQuickItem', objectName='tabsModelProperties')

        if tabs_model_properties.selectedIndex == index:
            return

        popover = self.get_root_instance().select_single(
            objectName='tabsPopover')

        try:
            # 1.3, using ActionSelectionPopover
            action_name = 'select_tab_' + str(index)
            popover.click_action_button(action_name)
        except _common.ToolkitException:
            try:
                # < 1.3 using custom popover
                tab_button = self.get_root_instance().select_single(
                    objectName='tabButton' + str(index))
                self.pointing_device.click_object(tab_button)
            except dbus.StateNotFoundError:
                raise _common.ToolkitException(
                    "Tab button {0} not found.".format(index))

        self.wait_for_animation()

    def click_action_button(self, action_object_name):
        """Click an action button of the header.

        :parameter object_name: The QML objectName property of the action
        :raise ToolkitException: If there is no action button with that object
            name.

        """
        self.ensure_visible()

        try:
            # for Lomiri.Components 1.3
            actionbar = self.select_single(
                'ActionBar', objectName='headerActionBar')
            actionbar.click_action_button(action_object_name)
        except dbus.StateNotFoundError:
            # for Lomiri.Components < 1.3
            button = self._get_action_button(action_object_name)
            # In an animating header, the button is disabled until the header
            #   animation is done. Wait for that:
            button.enabled.wait_for(True)
            self.pointing_device.click_object(button)
        except _common.ToolkitException:
            # Catch 'Button not found in ActionBar or overflow' exception
            raise _common.ToolkitException(
                'Button not found in header or overflow')

        self.wait_for_animation()

    def _get_action_button(self, action_object_name):
        self.wait_for_animation()
        try:
            object_name = action_object_name + "_header_button"
            button = self.select_single(objectName=object_name)
        except dbus.StateNotFoundError:
            # the button is not in the header, but it may be in the overflow
            try:
                button = self._get_action_button_in_overflow(
                    action_object_name)
            except dbus.StateNotFoundError:
                raise _common.ToolkitException(
                    'Button not found in header or overflow')

        return button

    def _get_action_button_in_overflow(self, action_object_name):
        actions_overflow_button = self.select_single(
            objectName='actions_overflow_button')

        if not actions_overflow_button.visible:
            raise _common.ToolkitException('No actions in overflow')

        # open the popover
        self.pointing_device.click_object(actions_overflow_button)
        object_name = action_object_name + "_header_overflow_button"

        # the popover is not a child of the header, so use the root object
        # to find the requested button
        return self.get_root_instance().select_single(objectName=object_name)


class Header(AppHeader):
    """Autopilot helper for the deprecated Header."""

    def __init__(self, *args):
        logger.warning(
            'Header is an internal QML component of Lomiri.Components and '
            'its API may change or be removed at any moment. Please use '
            'MainView and Page instead.')
        super().__init__(*args)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Lomiri UI Toolkit Autopilot custom proxy objects."""

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus
from autopilot import introspection

from lomiriuitoolkit._custom_proxy_objects import (
    _common,
    popups,
    _tabs,
    _toolbar,
)


_NO_TABS_ERROR = 'The MainView has no Tabs.'
_NO_TOOLBAR_ERROR = 'The MainView has no Toolbar.'


logger = logging.getLogger(__name__)


class MainView(_common.LomiriUIToolkitCustomProxyObjectBase):
    """MainView Autopilot custom proxy object."""

    def __init__(self, *args):
        super().__init__(*args)
        if not hasattr(self, 'useDeprecatedToolbar'):
            self.useDeprecatedToolbar = None

    @classmethod
    def validate_dbus_object(cls, path, state):
        if super().validate_dbus_object(path, state):
            # This covers MainView 0.1/1.0/1.1 and possible components
            # derived from MainView (e.g. "CustomMainView") that have
            # their own CPO.
            # Using objectName for selecting a MainView is recommneded.
            return True

        name = introspection.get_classname_from_path(path)
        if name == b'MainView12':
            # MainView 1.2. Must be selected using objectName.
            return True

        return False

    def get_header(self):
        """Return the AppHeader custom proxy object of the MainView."""
        try:
            return self.select_single(objectName='MainView_Header')
        except dbus.StateNotFoundError:
            raise _common.ToolkitException('The main view has no header.')

    def get_toolbar(self):
        """Return the Toolbar custom proxy object of the MainView.

        :raise ToolkitException: If the main view has no toolbar.

        """
        try:
            return self.select_single(_toolbar.Toolbar)
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(_NO_TOOLBAR_ERROR)

    @autopilot_logging.log_action(logger.info)
    def open_toolbar(self):
        """Open the toolbar if it is not already opened.

        :return: The toolbar.
        :raise ToolkitException: If the main view has no toolbar.

        """
        return self.get_toolbar().open()

    @autopilot_logging.log_action(logger.info)
    def close_toolbar(self):
        """Close the toolbar if it is opened.

        :raise ToolkitException: If the main view has no toolbar.

        """
        self.get_toolbar().close()

    def get_tabs(self):
        """Return the Tabs custom proxy object of the MainView.

        :raise ToolkitException: If the main view has no tabs.

        """
        try:
            return self.select_single(_tabs.Tabs)
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(_NO_TABS_ERROR)

    @autopilot_logging.log_action(logger.info)
    def switch_to_next_tab(self):
        """Open the next tab.

        :return: The newly opened tab.

        """
        logger.debug('Switch to next tab.')
        self.get_header().switch_to_next_tab()
        current_tab = self.get_tabs().get_current_tab()
        current_tab.visible.wait_for(True)
        return current_tab

    @autopilot_logging.log_action(logger.info)
    def switch_to_tab_by_index(self, index):
        """Open a tab.

        :parameter index: The index of the tab to open.
        :return: The newly opened tab.
        :raise ToolkitException: If the tab index
                is out of range.

        """
        if self.useDeprecatedToolbar:
            return self._switch_to_tab_in_deprecated_tabbar_by_index(index)
        else:
            return self._switch_to_tab_in_drawer_by_index(index)

    def _switch_to_tab_in_deprecated_tabbar_by_index(self, index):
        tabs = self.get_tabs()
        number_of_tabs = tabs.get_number_of_tabs()
        if index >= number_of_tabs:
            raise _common.ToolkitException('Tab index out of range.')

        current_tab = tabs.get_current_tab()
        number_of_switches = 0
        while not tabs.selectedTabIndex == index:
            logger.debug(
                'Current tab index: {0}.'.format(tabs.selectedTabIndex))
            if number_of_switches >= number_of_tabs - 1:
                # This prevents a loop. But if this error is ever raised, it's
                # likely there's a bug on the helper or on the QML Tab.
                raise _common.ToolkitException(
                    'The tab with index {0} was not selected.'.format(index))
            current_tab = self.switch_to_next_tab()
            number_of_switches += 1
        return current_tab

    def _switch_to_tab_in_drawer_by_index(self, index):
        tabs = self.get_tabs()
        number_of_tabs = tabs.get_number_of_tabs()
        if index >= number_of_tabs:
            raise _common.ToolkitException('Tab index out of range.')

        if index != tabs.selectedTabIndex:
            self.get_header().switch_to_tab_by_index(index)
        current_tab = tabs.get_current_tab()
        return current_tab

    @autopilot_logging.log_action(logger.info)
    def switch_to_previous_tab(self):
        """Open the previous tab.

        :return: The newly opened tab.

        """
        tabs = self.get_tabs()
        if tabs.selectedTabIndex == 0:
            previous_tab_index = tabs.get_number_of_tabs() - 1
        else:
            previous_tab_index = tabs.selectedTabIndex - 1
        return self.switch_to_tab_by_index(previous_tab_index)

    @autopilot_logging.log_action(logger.info)
    def switch_to_tab(self, object_name):
        """Open a tab.

        :parameter object_name: The QML objectName property of the tab.
        :return: The newly opened tab.
        :raise ToolkitException: If there is no tab with that object
            name.

        """
        tabs = self.get_tabs()
        for index, tab in enumerate(tabs.select_many('Tab')):
            if tab.objectName == object_name:
                return self.switch_to_tab_by_index(tab.index)
        raise _common.ToolkitException(
            'Tab with objectName "{0}" not found.'.format(object_name))

    def get_action_selection_popover(self, object_name):
        """Return an ActionSelectionPopover custom proxy object.

        :parameter object_name: The QML objectName property of the popover.

        """
        return self.wait_select_single(
            popups.ActionSelectionPopover, objectName=object_name)

    def get_text_input_context_menu(self, object_name):
        """Return a TextInputContextMenu emulator.

        :parameter object_name: The QML objectName property of the popover.

        """
        return self.wait_select_single(
            popups.TextInputPopover, objectName=object_name)

    @autopilot_logging.log_action(logger.info)
    def go_back(self):
        """Go to the previous page."""
        if self.useDeprecatedToolbar:
            toolbar = self.open_toolbar()
            toolbar.click_back_button()
        else:
            self.get_header().click_back_button()

    @autopilot_logging.log_action(logger.info)
    def click_action_button(self, action_object_name):
        """Click the specified button.

        :parameter action_object_name: the objectName of the action to trigger.
        :raise ToolkitException: The requested button is not available.

        """
        if self.useDeprecatedToolbar:
            raise _common.ToolkitException(
                "App is using deprecated toolbar instead of new header")
        else:
            header = self.open_header()
            header.click_action_button(action_object_name)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
from testtools.matchers import Equals
from autopilot.matchers import Eventually
from lomiriuitoolkit import tests


class TouchAdaptorTestCase(tests.QMLFileAppTestCase):
    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_touchadaptor.qml')

    def get_command_line(self, command_line):
        command_line.append('-touch')
        return command_line

    def test_apparent_touch_screen(self):
        touchArea = self.main_view.select_single(objectName="touchArea")
        label = self.main_view.select_single(objectName="label")
        # Trigger handlers as a courtesy to a watching person
        self.pointing_device.click_object(touchArea)
        self.assertThat(label.text, Eventually(Equals("Touched")))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the CheckBox Qml component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class EnabledCheckBoxTests(TavastiaTestCase):
    """Tests for an enabled CheckBox component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"EnabledCheckBoxTests")

    def test_can_select_checkbox(self):
        """Must be able to select the Qml CheckBox component."""

        obj = self.app.select_single('CheckBox')
        self.assertThat(obj, Not(Is(None)))

    def test_clicked_signal_emitted(self):
        """Clicking the checkbox component must emit the clicked() signal."""

        obj = self.app.select_single('CheckBox')
        signal = obj.watch_signal('clicked(QVariant)')

        self.assertThat(obj.checked, Eventually(Equals(False)))

        self.mouse.move_to_object(obj)
        self.mouse.click()

        self.assertThat(obj.checked, Eventually(Equals(True)))

        self.assertThat(signal.was_emitted, Equals(True))
        self.assertThat(signal.num_emissions, Equals(1))

class DisabledCheckBoxTests(TavastiaTestCase):
    """Tests for an enabled CheckBox component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"DisabledCheckBoxTests")

    def test_clicked_signal_not_emitted(self):
        """Clicking a disabled CheckBox must not emit the clicked() signal."""

        obj = self.app.select_single('CheckBox')
        signal = obj.watch_signal('clicked(QVariant)')

        self.assertThat(obj.checked, Equals(False))

        self.mouse.move_to_object(obj)
        self.mouse.click()

        self.assertThat(obj.checked, Eventually(Equals(False)))

        self.assertThat(signal.was_emitted, Equals(False))
        self.assertThat(signal.num_emissions, Equals(0))

